/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig;

import java.awt.Font;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URI;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.TransferHandler;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.plaf.FontUIResource;
import ssg.tools.easyconfig.data.NSetting;
import ssg.tools.easyconfig.data.NSettingSource;
import ssg.tools.easyconfig.data.handler.BaseConfigurationHandler;
import ssg.tools.easyconfig.data.handler.CompositeConfigurationHandler;
import ssg.tools.easyconfig.data.handler.IConfigurationHandler;
import ssg.tools.easyconfig.data.handler.TemplateConfigurationHandler;
import ssg.tools.easyconfig.models.ConfigurationChangeEvent;
import ssg.tools.easyconfig.models.ConfigurationChangeListener;
import ssg.tools.easyconfig.utils.IOUtils;
import ssg.tools.easyconfig.utils.Ring;
import ssg.tools.easyconfig.vc.BG_POSITION;
import ssg.tools.easyconfig.vc.ConfiguratorComponentsPanel;
import ssg.tools.easyconfig.vc.CreativeConfiguratorPanel;
import ssg.tools.easyconfig.vc.CreativePanel;
import ssg.tools.easyconfig.vc.DefaultConfiguratorPanel;
import ssg.tools.easyconfig.vc.IConfiguratorPanel;

/**
 * Experimental thus "unreadable" sample of configurator UI.
 * Allows us of combination of setting templates for multiple sources.
 * 
 * Planned though not realized yet to allow templates editing.
 * 
 * @author ssg
 */
public class EasyConfigurator extends javax.swing.JFrame implements IConfiguratorPanel, ConfigurationChangeListener {

    public static enum EC_FEATURE {

        drag_n_drop,
        components_view,
        default_view
    }
    public static final String TEMPLATE_EXTENSION = ".config.template";

    /**
     * Creates new form EasyConfigurator
     */
    public EasyConfigurator(String[] args) {
        //this.setContentPane(this.creativeContentPanel);
        //this.setUndecorated(true);
        initComponents();
        initCustom(args);
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        creativePanel1 = new ssg.tools.easyconfig.vc.CreativePanel();
        jScrollPaneConfigurator = new javax.swing.JScrollPane();
        jComboBoxConfigs = new javax.swing.JComboBox();
        jButtonNext = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        jScrollPaneConfigurator.setOpaque(false);

        jComboBoxConfigs.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        jComboBoxConfigs.setOpaque(false);

        jButtonNext.setText("Next view");
        jButtonNext.setOpaque(false);
        jButtonNext.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonNextActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout creativePanel1Layout = new javax.swing.GroupLayout(creativePanel1);
        creativePanel1.setLayout(creativePanel1Layout);
        creativePanel1Layout.setHorizontalGroup(
            creativePanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(creativePanel1Layout.createSequentialGroup()
                .addComponent(jComboBoxConfigs, 0, 436, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jButtonNext, javax.swing.GroupLayout.PREFERRED_SIZE, 127, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(88, 88, 88))
            .addGroup(creativePanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addComponent(jScrollPaneConfigurator, javax.swing.GroupLayout.DEFAULT_SIZE, 658, Short.MAX_VALUE))
        );
        creativePanel1Layout.setVerticalGroup(
            creativePanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, creativePanel1Layout.createSequentialGroup()
                .addGap(0, 472, Short.MAX_VALUE)
                .addGroup(creativePanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jComboBoxConfigs, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jButtonNext)))
            .addGroup(creativePanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(creativePanel1Layout.createSequentialGroup()
                    .addComponent(jScrollPaneConfigurator, javax.swing.GroupLayout.DEFAULT_SIZE, 468, Short.MAX_VALUE)
                    .addGap(29, 29, 29)))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(creativePanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(creativePanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jButtonNextActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonNextActionPerformed
        // TODO add your handling code here:
        JComponent c = views.get(ring.next());
        if (c instanceof IConfiguratorPanel) {
            configPanel = (IConfiguratorPanel) c;
        }
        this.jScrollPaneConfigurator.setViewportView((JComponent) configPanel);
        this.jButtonNext.setText("" + ring.peek() + " view");
        this.jButtonNext.repaint();
    }//GEN-LAST:event_jButtonNextActionPerformed

    /**
     * @param args the command line arguments
     */
    public static void main(final String args[]) {

        /*
         * Set the Nimbus look and feel
         */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /*
         * If Nimbus (introduced in Java SE 6) is not available, stay with the
         * default look and feel. For details see
         * http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(EasyConfigurator.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(EasyConfigurator.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(EasyConfigurator.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(EasyConfigurator.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>


        //////////// adjust resources
        // change font sizes
        if (1 == 0) {
            UIDefaults defaults = UIManager.getDefaults();
            Enumeration keys = defaults.keys();
            while (keys.hasMoreElements()) {
                int fontSize = 12;
                Object key = keys.nextElement();

                if ((key instanceof String) && (((String) key).endsWith(".font"))) {
                    //FontUIResource font = (FontUIResource) UIManager.get(key);
                    Font font = (Font) UIManager.get(key);
                    FontUIResource fuir = (font instanceof FontUIResource) ? (FontUIResource) font : null;
                    if (fuir != null) {
                        System.out.println("Original Font " + key + " = " + font);
                        defaults.put(key, new FontUIResource(font.getFontName(), font.getStyle(), font.getSize() + 4));
                        font = (Font) UIManager.get(key);
                        System.out.println("Modified Font " + key + " = " + font);
                    }
                }
            }
        }


        /*
         * Create and display the form
         */
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new EasyConfigurator(args).setVisible(true);
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private ssg.tools.easyconfig.vc.CreativePanel creativePanel1;
    private javax.swing.JButton jButtonNext;
    private javax.swing.JComboBox jComboBoxConfigs;
    private javax.swing.JScrollPane jScrollPaneConfigurator;
    // End of variables declaration//GEN-END:variables
    CreativeConfiguratorPanel creativeConfiguratorPanel = new CreativeConfiguratorPanel();
    DefaultConfiguratorPanel defaultConfiguratorPanel = new DefaultConfiguratorPanel();
    List<IConfigurationHandler> configs = new ArrayList<IConfigurationHandler>();
    IConfigurationHandler currentConfig;
    TemplateConfigurationHandler currentTemplate = new TemplateConfigurationHandler();
    ConfiguratorComponentsPanel ccp = new ConfiguratorComponentsPanel();
    IConfiguratorPanel configPanel = creativeConfiguratorPanel;
    Map<String, JComponent> views = new LinkedHashMap<String, JComponent>();
    Ring<String> ring = new Ring<String>();
    List<EC_FEATURE> features = new ArrayList<EC_FEATURE>();

    // com.ssg.tests.jsf.flexing.config.DefaultConfiguratorPanel
    void initCustom(String[] args) {
        // assign title
        String title = IOUtils.findParam(args, "-title");
        if (title == null) {
            title = "EasyConfig Tool";
        }
        setTitle(title);


        this.jScrollPaneConfigurator.setOpaque(false);
        this.jScrollPaneConfigurator.getViewport().setOpaque(false);
        this.creativeConfiguratorPanel.setOpaque(false);
        this.defaultConfiguratorPanel.setOpaque(false);
        this.creativePanel1.setOpaque(false);

//        this.jComboBoxConfigs.setEditable(true);
//        this.jComboBoxConfigs.setOpaque(false);
//        ((JTextField) this.jComboBoxConfigs.getEditor().getEditorComponent()).setOpaque(false);


        //this.jScrollPaneConfigurator.setBorder(null);
        this.jScrollPaneConfigurator.setViewportBorder(null);
        this.jScrollPaneConfigurator.getViewport().setBorder(null);
        this.creativeConfiguratorPanel.setBorder(null);
        this.defaultConfiguratorPanel.setBorder(null);
        this.creativePanel1.setBorder(null);
        this.jScrollPaneConfigurator.getViewport().getInsets().set(0, 0, 0, 0);
        this.jScrollPaneConfigurator.getInsets().set(0, 0, 0, 0);
        this.creativeConfiguratorPanel.getInsets().set(0, 0, 0, 0);
        this.defaultConfiguratorPanel.getInsets().set(0, 0, 0, 0);
        this.creativePanel1.getInsets().set(0, 0, 0, 0);

        this.creativePanel1.setBgAlpha(0.3f);



        String bgImage = IOUtils.findParam(args, "-bgImage");
        //bgImage="";
        if (bgImage != null) {
            try {
                BG_POSITION pos = null;
                ImageIcon img = null;
                if (bgImage.isEmpty()) {
                    img = null;//new ImageIcon(getClass().getClassLoader().getResource("" + getClass().getPackage().getName().replace(".", "/") + "/data/ssg_transparent.png"));
                } else {
                    String[] ss = bgImage.split(",");
                    img = new ImageIcon(IOUtils.string2uri(ss[0].trim()).toURL());
                    if (ss.length > 1) {
                        try {
                            pos = BG_POSITION.valueOf(ss[1].trim());
                        } catch (Throwable th) {
                            th.printStackTrace();
                        }
                    }
                }
                if (img != null) {
                    this.creativePanel1.setBgImage(img, pos);
                    this.defaultConfiguratorPanel.setBgImage(img, pos);
                }

            } catch (Throwable th) {
                th.printStackTrace();
            }
        }

        ccp.setTemplate(currentTemplate);

        this.jComboBoxConfigs.setModel(new DefaultComboBoxModel(new Vector(configs)));
        this.jComboBoxConfigs.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                if (ItemEvent.SELECTED == e.getStateChange()) {
                    IConfigurationHandler config = (IConfigurationHandler) e.getItem();
                    setConfig(config);
                }
            }
        });

        this.ccp.addConfigurationChangeListener(this);

        initDefaultFeatures();
        handleFeatureParameters(args);
        for (EC_FEATURE feature : features) {
            switch (feature) {
                case drag_n_drop:
                    this.setTransferHandler(new TransferHandler() {

                        List<String> dfns = new ArrayList<String>() {

                            {
                                add("application/x-java-url");
                                add("text/plain");
                                add("application/x-java-file-list");
                            }
                        };

                        @Override
                        public boolean importData(TransferHandler.TransferSupport support) {
                            System.out.println("importData(" + support + ")");
                            for (DataFlavor df : support.getDataFlavors()) {
                                String dfn = df.getHumanPresentableName();
                                System.out.print("import DF: " + dfn + ": ");
                                try {
                                    System.out.println(support.getTransferable().getTransferData(df));
                                } catch (Throwable th) {
                                    System.out.println("ERROR: " + th);
                                }

                                if (dfns.contains(dfn)) {
                                    try {
                                        Object o = support.getTransferable().getTransferData(df);
                                        if (o instanceof String) {
                                            System.out.println("import: " + df.getHumanPresentableName());
                                            System.out.println("... [" + o.getClass() + "]" + ((o instanceof String) ? o : "<>"));
                                            checkIfConfig(o);
                                        }
                                        if (o instanceof List) {
                                            for (Object oo : (List) o) {
                                                System.out.println("import(" + ((List) o).indexOf(oo) + "): " + df.getHumanPresentableName());
                                                if (oo instanceof String || oo instanceof URL || oo instanceof File) {
                                                    System.out.println("... ... [" + oo.getClass() + "]" + oo);
                                                    checkIfConfig(oo);
                                                }
                                            }
                                        }
                                    } catch (UnsupportedFlavorException ufex) {
                                    } catch (IOException ioex) {
                                    }
                                }
                            }
                            return true;
                        }

                        @Override
                        public boolean importData(JComponent comp, Transferable t) {
                            System.out.println("importData(" + comp.getClass().getSimpleName() + ", " + t + ")");
                            for (DataFlavor df : t.getTransferDataFlavors()) {
                                System.out.println("import: " + df.getHumanPresentableName());
                            }
                            return true;
                        }

                        @Override
                        public boolean canImport(TransferHandler.TransferSupport support) {
                            //System.out.println("canImport(" + support + ")");
                            for (DataFlavor df : support.getDataFlavors()) {
                                //System.out.println("canImport: " + df.getHumanPresentableName());
                            }
                            return support.isDrop();
                        }

                        @Override
                        public boolean canImport(JComponent comp, DataFlavor[] transferFlavors) {
                            //System.out.println("canImport(" + comp.getClass().getSimpleName() + ", " + transferFlavors + ")");
                            for (DataFlavor df : transferFlavors) {
                                //System.out.println("canImport: " + df.getHumanPresentableName());
                            }
                            return true;
                        }
                    });
                    break;
                case default_view:
                    views.put("Edit", defaultConfiguratorPanel);
                    break;
                case components_view:
                    views.put("Components", ccp);
                    break;
            }
        }

        if (views.isEmpty()) {
            views.put("Edit", defaultConfiguratorPanel);
        }
        ring.addAll(views.keySet());
        jButtonNextActionPerformed(null);
        if (views.size() < 2) {
            this.jButtonNext.setVisible(false);
        } else {
            this.jButtonNext.setVisible(true);
        }


        handleDataParameters(args);
    }

    boolean checkIfConfig(Object obj) {
        System.out.println("Check if supported configuration source for " + obj);
        {
            ImageIcon img = null;
            try {
                if (obj instanceof File) {
                    img = new ImageIcon(((File) obj).toURI().toURL());
                } else if (obj instanceof URL) {
                    img = new ImageIcon((URL) obj);
                } else if (obj instanceof String) {
                    img = new ImageIcon(IOUtils.string2uri((String) obj).toURL());
                }
            } catch (Throwable th) {
            }
            if (img != null && img.getIconWidth() > 0 && img.getIconHeight() > 0) {
                this.creativePanel1.setBgImage(img, null);
                this.defaultConfiguratorPanel.setBgImage(img, null);
                this.repaint();
                return false;
            }
        }

        try {
            BaseConfigurationHandler config = null;
            if (obj instanceof File) {
                File file = (File) obj;
                if (file.isFile() && file.getName().endsWith(TEMPLATE_EXTENSION)) {
                    if (currentConfig != null) {
                        IConfigurationHandler theConfig = currentConfig;
                        boolean modified = false;
                        try {
                            List<NSettingSource> sources = theConfig.loadConfigurationTemplate(new FileInputStream(file), "*");
                            for (NSettingSource exss : sources) {
                                for (NSettingSource ngss : theConfig.getSettings()) {
                                    if (exss.getPath().equals(ngss.getPath())) {
                                        if (exss.getType().equals(ngss.getType())) {
                                            for (NSetting es : exss.getSettings()) {
                                                for (NSetting ns : ngss.getSettings()) {
                                                    if (es.getLocator() != null && es.getLocator().equals(ns.getLocator())) {
                                                        if (es.getValue() != null && ns.isDifferent(es.getValue())) {
                                                            ns.setValue(es.getValue());
                                                            modified = true;
                                                        }
                                                        break;
                                                    }
                                                }
                                            }
                                        } else {
                                            System.err.println("Ignore settings for " + exss.getPath() + ": ext type=" + exss.getType() + ", int type=" + ngss.getType());
                                        }
                                    }
                                }
                            }
                        } catch (IOException ioex) {
                            ioex.printStackTrace();
                        }
                        if (modified) {
                            setConfig(theConfig);
                        }
                    } else {
                        try {
                            currentTemplate.setSettings(currentTemplate.loadConfigurationTemplate(new FileInputStream(file), "*"), true);
                            currentTemplate.getTemplates().add(file.toURI());
                            this.ccp.setTemplate(currentTemplate);
                            this.ccp.updateTemplate();
                        } catch (IOException ioex) {
                            ioex.printStackTrace();
                        }
                    }
                    return false;
                }
                //config = new FileConfigurationHandler(file);
                config = new CompositeConfigurationHandler(file);
            } else if (obj instanceof URL) {
                //config = new FileConfigurationHandler((URL) obj);
                config = new CompositeConfigurationHandler((URL) obj);
            } else if (obj instanceof String) {
                File f = null;
                try {
                    f = new File((String) obj);
                    //config = new FileConfigurationHandler(f);
                    config = new CompositeConfigurationHandler(f);
                } catch (Throwable th) {
                }
                if (config == null) {
                    URL u = null;
                    try {
                        u = new URL((String) obj);
                        //config = new FileConfigurationHandler(u);
                        config = new CompositeConfigurationHandler(u);
                    } catch (Throwable th) {
                    }
                }
            }
            if (config != null) {
                // override settings if needed...
                if (!currentTemplate.getSettings().isEmpty()) {
                    try {
                        File tempFile = File.createTempFile("_tmp_", TEMPLATE_EXTENSION);
                        tempFile.deleteOnExit();
                        System.out.println("Temp template: " + tempFile);
                        config.saveConfigurationTemplate(currentTemplate.getSettings(), new FileOutputStream(tempFile), "*");
                        config.setSettings(config.loadConfigurationTemplate(new FileInputStream(tempFile), "*"), true);
                        tempFile.delete();
                        //config.loadSettings();
                    } catch (IOException ioex) {
                        ioex.printStackTrace();
                    }
                } else {
                    config.initSettingsList();
                }
                try {
                    config.loadSettings();
                } catch (IOException ioex) {
                    ioex.printStackTrace();
                }
                if (!configs.contains(config)) {
                    configs.add(config);
                    configsListChanged();
                    if (configs.size() > 0) {
                        setConfig(config);
                    }
                    return true;
                }
            }
        } catch (IOException ioex) {
        }
        return false;
    }

    void configsListChanged() {
        this.jComboBoxConfigs.setModel(new DefaultComboBoxModel(new Vector(configs)));
        this.jComboBoxConfigs.revalidate();
        this.jComboBoxConfigs.repaint();
    }

    public void setConfig(IConfigurationHandler config) {
        currentConfig = config;
        this.ccp.setConfig(currentConfig);
        this.creativeConfiguratorPanel.setConfig(currentConfig);
        this.defaultConfiguratorPanel.setConfig(currentConfig);
        ((JComponent) configPanel).repaint();
    }

    public IConfigurationHandler getConfig() {
        return currentConfig;
    }

    public void configurationChanged(ConfigurationChangeEvent e) {
        if (this.equals(e.getSource())) {
            return;
        }
        if (e.getChangedSource() != null) {
            if (!this.configs.contains(e.getChangedSource())) {
                configs.add(e.getChangedSource());
                configsListChanged();
            }
            setConfig(e.getChangedSource());
        }
    }

    public void initDefaultFeatures() {
        features.add(EC_FEATURE.drag_n_drop);
        features.add(EC_FEATURE.default_view);
        features.add(EC_FEATURE.components_view);
    }

    public void handleFeatureParameters(String[] args) {
        List<String> featureNames = IOUtils.findParamValues(args, "-feature");
        for (String fn : featureNames) {
            if (fn == null || fn.isEmpty()) {
                continue;
            }
            boolean add = true;
            if (fn.startsWith("-")) {
                add = false;
                fn = fn.substring(1).trim();
            } else if (fn.startsWith("+")) {
                add = true;
                fn = fn.substring(1).trim();
            }
            if (fn.isEmpty()) {
                continue;
            }
            String[] ss = fn.split(",");
            for (String n : ss) {
                n = n.trim();
                if (n.isEmpty()) {
                    continue;
                }
                if ("all".equals(n)) {
                    if (add) {
                        for (EC_FEATURE f : EC_FEATURE.values()) {
                            if (!features.contains(f)) {
                                features.add(f);
                            }
                        }
                    } else {
                        features.clear();
                    }
                } else {
                    try {
                        EC_FEATURE f = EC_FEATURE.valueOf(n);
                        if (add) {
                            if (!features.contains(f)) {
                                features.add(f);
                            }
                        } else {
                            if (features.contains(f)) {
                                features.remove(f);
                            }
                        }
                    } catch (Throwable th) {
                    }
                }
            }
        }
    }

    public void handleDataParameters(String[] args) {
        List<String> templateNames = IOUtils.findParamValues(args, "-template");
        List<String> configNames = IOUtils.findParamValues(args, "-source");
        for (String template : templateNames) {
            URI uri = IOUtils.string2uri(template);
            if (uri != null) {
                ccp.getTemplate().getTemplates().add(uri);
            } else {
                System.out.println("Ignored template '" + template + "': failed to convert to URI.");
            }
        }

        try {
            ccp.getTemplate().loadSettings();
            ccp.updateTemplate();
        } catch (IOException ioex) {
            System.err.println("Error while loading template settings. Ignored. ERROR: " + ioex);
        }

        for (String config : configNames) {
            URI uri = IOUtils.string2uri(config);
            if (uri != null) {
                try {
                    IConfigurationHandler ch = new CompositeConfigurationHandler(uri);
                    ccp.setConfig(ch);
                } catch (IOException ioex) {
                    System.out.println("Ignored source '" + config + "': failed to convert to URI.");
                }
            } else {
                System.out.println("Ignored source '" + config + "': failed to convert to URI.");
            }
        }
    }

    public void help() {
        StringBuilder sb=new StringBuilder();
        sb.append("Configuration tool to read/apply settings distributed over multiple sources.\n");
        sb.append("Usage: "+getClass().getName()+" [-template=<file|url> ...][-source=<file|url> ...] [-feature=[-|+]<feature name> ...] [-<option name>[=<option v<lue> ...]\n");
        sb.append(" * Template is groupped properties file defining type, path, and settings (see provided examples).\n");
        sb.append(" * Source is folder or file or URL. Generakky setting path is path relative to source folder or, if source if archive file - path inside archive.\n");
        sb.append(" * Feature is named predefined property used to adjust UI.\n");
        sb.append("     Feature name may be prefixed with '-' to be disabled or '+' to be enabled (default).\n");
        sb.append("     Special feature name 'all' allows to apply enable/disable to all features.\n");
        sb.append("     Following features are available:\n");
        sb.append("     - drag_n_drop : enable drag-n-drop to add templates/sources\n");
        sb.append("     - components_view : enable components view, i.e. view where current set of templates and source is presented in detail.\n");
        sb.append("     - default_view : enable default view with tree-like presentation of settings and table to edit them.\n");
        sb.append(" * Options are used to provide/override defaults used in UI. they are:\n");
        sb.append("   - title : assigns main window title text\n");
        sb.append("   - bgImage : assigns background image, Default - no image.\n");
        sb.append("");
        sb.append("");
        sb.append("");
        System.out.println(sb.toString());
    }
    
    public CreativePanel getCreativePanel() {
        return this.creativePanel1;
    }
}
