/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.data.handler;

import ssg.tools.easyconfig.data.NSettingSource;
import ssg.tools.easyconfig.utils.IOUtils;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * Provides access to settings list.
 *
 * @author ssg
 */
public class TemplateConfigurationHandler extends BaseConfigurationHandler {

    public static final String TEMPLATE_EXTENSION = ".ctmpl";
    private URI uri;
    private List<URI> templates = new ArrayList<URI>();
    private Map<String, Map<String, String>> options = new LinkedHashMap<String, Map<String, String>>();

    public TemplateConfigurationHandler() {
    }

    public TemplateConfigurationHandler(URI uri) {
        this.uri = uri;
    }

    public String getLocation() {
        return (getUri() != null) ? getUri().toString() : "undefined";
    }

    public void initSettingsList() {
        //throw new UnsupportedOperationException("Not supported yet.");
    }

    public void loadSettings() throws IOException {
        if (getUri() != null) {
            Map<String, Map<String, String>> maps = IOUtils.loadTextConfiguration(IOUtils.uri2InputStream(getUri(), null), templateEncoding);
            Map<String, String> map = maps.get("templates");
            if (map != null) {
                for (String key : map.keySet()) {
                    try {
                        URI tURI = new URI(map.get(key));
                        getTemplates().add(tURI);
                    } catch (URISyntaxException usex) {
                    }
                }
                maps.remove("templates");
            }
            getOptions().putAll(maps);
        }
        for (URI template : getTemplates()) {
            List<NSettingSource> ss = this.loadConfigurationTemplate(IOUtils.uri2InputStream(template, null), "*");
            this.setSettings(ss, false);
        }

        setLastLoaded(System.currentTimeMillis());
    }

    public void saveSettings() throws IOException {
        if (getUri() != null) {
            OutputStream os = IOUtils.uri2OutputStream(getUri(), null);
            Writer wr = new OutputStreamWriter(os, templateEncoding);
            wr.write("#composed template, saved at " + (new Date()) + "\n");

            wr.write("[templates]\n");
            for (URI uri : getTemplates()) {
                wr.write("uri=" + uri + "\n");
            }
            for (String key : getOptions().keySet()) {
                wr.write("\n[" + key + "]\n");
                Map<String, String> map = getOptions().get(key);
                for (String on : map.keySet()) {
                    wr.write("" + on + " = " + map.get(on) + "\n");
                }
            }
            wr.close();
        }
    }

    /**
     * @return the uri
     */
    public URI getUri() {
        return uri;
    }

    /**
     * @param uri the uri to set
     */
    public void setUri(URI uri) {
        this.uri = uri;
    }

    /**
     * @return the templates
     */
    public List<URI> getTemplates() {
        return templates;
    }

    /**
     * @param templates the templates to set
     */
    public void setTemplates(List<URI> templates) {
        this.templates = templates;
        setLastModified(System.currentTimeMillis());
    }

    /**
     * @return the options
     */
    public Map<String, Map<String, String>> getOptions() {
        return options;
    }

    /**
     * @param options the options to set
     */
    public void setOptions(Map<String, Map<String, String>> options) {
        this.options = options;
        setLastModified(System.currentTimeMillis());
    }

    public List<NSettingSource> cloneSettings() throws IOException {
        ByteArrayOutputStream bos = null;
        ByteArrayInputStream bis = null;
        try {
            bos = new ByteArrayOutputStream();
            saveConfigurationTemplate(getSettings(), bos, "*");
            bis = new ByteArrayInputStream(bos.toByteArray());
            return loadConfigurationTemplate(bis, "*");
        } finally {
            if (bos != null) {
                bos.close();
            }
            if (bis != null) {
                bis.close();
            }
        }
    }
}
