/*
 * Copyright 2012 sesidoro.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.data.handler.support;

import java.io.IOException;
import java.net.URI;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;
import ssg.tools.easyconfig.data.APPLY_STRATEGY;

/**
 * Uses object as uri shcema to provide access to runtime properties of object defined by name in path.
 * Objects referred by URI path must be registered with "set" method and may be unregistered with "set" method with null value.
 * 
 * This hanler works in pair with "object" source where locator is expression supported in class P.
 * 
 * @author ssg
 */
public class ObjectSourceHandler implements ISourceHandler {

    static Map<String, Object> objectsMap = new LinkedHashMap<String, Object>();
    ////////////////////////////////////////////////////////////////////////////
    Object owner;
    URI inputURI;
    URI outputURI;
    Object input;
    Object output;
    // vars for opened state. Opened if hasNext!=null.
    Boolean readOnly = null;
    Boolean hasNext = null;
    APPLY_STRATEGY strategy;

    public ObjectSourceHandler(URI input) {
        this.inputURI = input;
        this.input = uri2object(this.inputURI);
        this.output = this.input;
    }

    public ObjectSourceHandler(URI input, URI output) {
        this.inputURI = input;
        this.outputURI = output;
        this.input = uri2object(this.inputURI);
        this.output = uri2object(this.outputURI);
        if (this.output == null) {
            this.output = this.input;
        }
    }

    public void init(Object owner) throws IOException {
        this.owner = owner;
    }

    public void close() throws IOException {
        hasNext = null;
        readOnly = null;
        strategy = null;
    }

    /**
     * Not applicable: can copy only values defined in settings and here we can
     * get if setting is not defined.
     *
     * @throws IOException
     */
    public void copyInputToOutput() throws IOException {
    }

    public Object getInput() throws IOException {
        return (hasNext != null) ? input : null;
    }

    public String getName() {
        return (inputURI != null) ? inputURI.getPath() : null;
    }

    public URI getUri() {
        return inputURI;
    }

    public Object getOutput() throws IOException {
        return (readOnly != null && !readOnly) ? output : null;
    }

    public boolean hasNext() {
        return hasNext != null && hasNext;
    }

    public void next() throws IOException {
        if (hasNext != null) {
            hasNext = false;
        } else {
            throw new IOException("next() on closed source.");
        }
    }

    public void open(boolean readOnly, APPLY_STRATEGY applyStrategy) throws IOException {
        hasNext = true;
        this.readOnly = readOnly;
        strategy = applyStrategy;
    }

    public void prepareForWrite(boolean modified) throws IOException {
    }

    public boolean canRead(Object obj) throws IOException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public ISourceHandler instanceFor(URI uriIn, URI uriOut) {
        return new ObjectSourceHandler(uriIn, uriOut);
    }

    /////////////////////////////////////////////////////////// utilities
    static Object uri2object(URI path) {
        String s = (path != null) ? path.toString() : "";
        return (s != null) ? objectsMap.get(s) : null;
    }

    /**
     * Assigns/removes path association with object.
     *
     * @param path
     * @param obj
     */
    public static void set(String path, Object obj) {
        if (path == null) {
            return;
        }
        if (obj != null) {
            objectsMap.put(path, obj);
        } else if (objectsMap.containsKey(path)) {
            objectsMap.remove(path);
        }
    }

    /**
     * Returns list of all registered object paths...
     *
     * @return
     */
    public static Collection<String> list() {
        return objectsMap.keySet();
    }
}
