/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.data.sources;

import ssg.tools.easyconfig.utils.NFactoryPrototype;
import ssg.tools.easyconfig.data.NSettingSource;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.util.Properties;
import java.util.ServiceLoader;

/**
 *
 * @author ssg
 */
public class NSettingSourceFactory extends NFactoryPrototype<NSettingSource> {

    private static ServiceLoader<NSettingSource> nsettingsLoader = ServiceLoader.load(NSettingSource.class);
    static NSettingSourceFactory factory;

    public static NSettingSourceFactory getInstance() {
        if (factory == null) {
            factory = new NSettingSourceFactory();
            factory.initDefaultTypes(true);
        }
        return factory;
    }

    @Override
    public boolean matchType(NSettingSource type, Object obj, boolean exact) {
        return false;
    }

    public NSettingSource newSource(String type, String path) {
        NSettingSource inst = getType(type, null);
        if (inst != null) {
            return inst.instanceFor(path);
        }
        return null;
    }

    @Override
    public void loadTypeDefinitions(InputStream is) throws IOException {
        LineNumberReader lnr = new LineNumberReader(new InputStreamReader(new BufferedInputStream(is), "UTF-8"));
        String section = "";
        Properties sectionProperties = null;

        for (NSettingSource nss : nsettingsLoader) {
            //System.out.println("setting source type: " + nss);
            types.put(nss.getType(), nss);
        }

        String s = null;
        while ((s = lnr.readLine()) != null) {
            s = s.trim();
            if (s.startsWith("#") || s.startsWith("//")) {
                continue;
            }
            if (s.startsWith("[")) {
                section = s.substring(1, s.length() - 2).trim().toLowerCase();
                if (section.isEmpty()) {
                    sectionProperties = null;
                } else {
                    sectionProperties = options.get(section);
                    if (sectionProperties == null) {
                        sectionProperties = new Properties();
                        options.put(s, sectionProperties);
                    }
                }
            }
            int idx = s.indexOf("=");
            if (idx == -1) {
                continue;
            }
            String name = s.substring(0, idx);
            s = s.substring(idx + 1);
            if (sectionProperties != null) {
                sectionProperties.put(name, s);
            }
        }
        lnr.close();
        
    }
}
