/*
 * Copyright 2012 sesidoro.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.data.sources;

import java.io.IOException;
import ssg.tools.easyconfig.data.NSetting;
import ssg.tools.easyconfig.data.NSettingSource;
import ssg.tools.easyconfig.data.types.ISettingType;
import ssg.tools.easyconfig.data.types.SettingTypeFactory;
import ssg.tools.easyconfig.utils.P;
import ssg.tools.easyconfig.utils.P.PP;

/**
 *
 * @author ssg
 */
public class ObjectSource extends NSettingSource implements IWritableSource {

    public static final String SOURCE_TYPE = "object";
    ////////////////////////////////////////////////////////////////////////////
    public PP p = new PP(P.DEFAULT_OPTIONS | P.EXPANSIONS);

    public ObjectSource() {
        super(null, SOURCE_TYPE);
    }

    public ObjectSource(String path) {
        super(path, SOURCE_TYPE);
    }

    @Override
    public boolean save(Object input, Object output) throws IOException {
        Object in=p.get(input, getPath());
        Object out= (output!=null) ? p.get(output, getPath()) : in;
        for (NSetting ns : getSettings()) {
            ns.setSaveError(null);
            if (out != null && p.exists(out, ns.getLocator())) {
                ISettingType st = SettingTypeFactory.getInstance().getType(ns.getType(), null);
                Object value = st.asValue(ns.getValue());
                try {
                    p.set(out, ns.getLocator(), value);
                    ns.setLoadedValue(ns.getValue());
                } catch (Throwable th) {
                    ns.setSaveError(th);
                }
            } else {
                if (output == null) {
                    ns.setSaveError(new Exception("No object for path '" + getPath() + "' to save value."));
                } else {
                    ns.setSaveError(new Exception("Location '" + ns.getLocator() + "' is not found in object " + output + " for path '" + getPath() + "'."));
                }
            }
        }
        setAppliedAt(System.currentTimeMillis());
        return true;
    }

    @Override
    public boolean load(Object input) throws IOException {
        Object in=p.get(input, getPath());
        for (NSetting ns : getSettings()) {
            ns.setLoadError(null);
            if (in != null && p.exists(in, ns.getLocator())) {
                // TODO: format loaded value using st info??
                ISettingType st = SettingTypeFactory.getInstance().getType(ns.getType(), null);
                Object objValue = p.get(in, ns.getLocator());
                if (objValue != null) {
                    String value = objValue.toString();
                    ns.setLoadedValue(value);
                    ns.setValue(value);
                    //System.out.println("   " + ns.getName() + " = " + ns.getLoadedValue());
                } else {
                    ns.setLoadedValue(null);
                    ns.setValue(null);
                }
            } else {
                if (input == null) {
                    ns.setLoadError(new Exception("No object for path '" + getPath() + "' to read value."));
                } else {
                    ns.setLoadError(new Exception("Location '" + ns.getLocator() + "' is not found in object " + input + " for path '" + getPath() + "'."));
                }
            }
        }
        setLoadedAt(System.currentTimeMillis());
        return true;
    }

    @Override
    public NSettingSource instanceFor(String path) {
        return new ObjectSource(path);
    }
}
