/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.data.sources;

import ssg.tools.easyconfig.data.NSettingSource;
import ssg.tools.easyconfig.data.NSetting;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Properties;

/**
 *
 * @author ssg
 */
public class PropertiesSettingSource extends NSettingSource implements IWritableSource<InputStream, OutputStream> { //IStreamSource {

    public static final String SOURCE_TYPE = "properties";

    public PropertiesSettingSource() {
        super(null, SOURCE_TYPE);
    }

    public PropertiesSettingSource(String path) {
        super(path, SOURCE_TYPE);
    }

    @Override
    public boolean load(InputStream is) throws IOException {
        Properties props = null;
        props = new Properties();
        //props.load(new InputStreamReader(zf.getInputStream(ze), "UTF-8"));
        props.load(new InputStreamReader(is, "UTF-8"));
        for (NSetting ns : getSettings()) {
            String value = props.getProperty(ns.getLocator());
            if (value != null) {
                ns.setLoadedValue(value);
                ns.setValue(value);
                //System.out.println("   " + ns.getName() + " = " + ns.getLoadedValue());
            }
        }
        setLoadedAt(System.currentTimeMillis());
        return true;
    }

    public boolean save(InputStream is, OutputStream os) throws IOException {
        LineNumberReader lnr = new LineNumberReader(new InputStreamReader(is, "UTF-8"));
        Writer wr = new OutputStreamWriter(new BufferedOutputStream(os), "UTF-8");
        String s = null;
        while ((s = lnr.readLine()) != null) {
            if (!s.trim().startsWith("#") && !s.trim().startsWith("//")) {
                String separator = "";
                // ensure valid '=' or ':' separator
                {
                    int idx0 = s.indexOf(':');
                    int idx1 = s.indexOf('=');
                    if (idx0 > 1) {
                        if (s.charAt(idx0 - 1) == '\\') {
                            idx0 = -1;
                        }
                    }
                    if (idx1 > 1) {
                        if (s.charAt(idx1 - 1) == '\\') {
                            idx1 = -1;
                        }
                    }
                    if (idx0 != -1 && idx1 != -1) {
                        if (idx0 < idx1) {
                            idx1 = -1;
                        } else {
                            idx0 = -1;
                        }
                    }
                    if (idx0 != -1) {
                        separator = ":";
                    } else if (idx1 != -1) {
                        separator = "=";
                    }
                }
                if (!separator.isEmpty()) {
                    String[] ss = s.split(separator);
                    if (ss.length > 1) {
                        String key = ss[0].trim();
                        String value = (ss.length > 1) ? ss[1] : "";
                        for (NSetting ns : getSettings()) {
                            //System.out.println("   " + ns.getName() + " [ro=" + ns.isReadOnly() + ", mod=" + ns.isModified() + "] = " + ns.getValue() + " / " + ns.getLoadedValue());
                            if (ns.getLocator().equals(key) && ns.isModified() && !ns.isReadOnly()) {
                                s = key + separator + " " + ns.getValue();
                                //System.out.println("   " + ns.getName() + " = " + ns.getLoadedValue());
                                break;
                            }
                        }
                    }
                }
            }
            wr.write(s + "\n");
        }
        wr.flush();
        // loaded value = value -> no modifications left.
        for (NSetting ns : getSettings()) {
            ns.setLoadedValue(ns.getValue());
            ns.setSaveError(null);
        }
        setAppliedAt(System.currentTimeMillis());
        return true;
    }

    @Override
    public NSettingSource instanceFor(String path) {
        if (path != null) {
            return new PropertiesSettingSource(path);
        } else {
            return null;
        }
    }
}
