/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.data.sources;

import ssg.tools.easyconfig.data.NSetting;
import ssg.tools.easyconfig.data.NSettingSource;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 *
 * @author ssg
 */
public class XMLSettingSource extends NSettingSource implements IWritableSource<InputStream, OutputStream> { // IStreamSource {

    public static final String SOURCE_TYPE = "xml";

    public XMLSettingSource() {
        super(null, SOURCE_TYPE);
    }

    public XMLSettingSource(String path) {
        super(path, SOURCE_TYPE);
    }

    @Override
    public boolean load(InputStream is) throws IOException {
        Document dom = null;
        try {
            DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
            domFactory.setNamespaceAware(false);
            DocumentBuilder builder = domFactory.newDocumentBuilder();
            //Document doc = builder.parse(zf.getInputStream(ze));
            Document doc = builder.parse(is);
            //zis.closeEntry();
            for (NSetting ns : getSettings()) {
                try {
                    XPath xpath = XPathFactory.newInstance().newXPath();
                    // XPath Query for showing all nodes value
                    XPathExpression expr = xpath.compile(ns.getLocator());
                    Object result = expr.evaluate(doc, XPathConstants.NODESET);
                    NodeList nodes = (NodeList) result;
                    for (int i = 0; i < nodes.getLength(); i++) {
                        String value = nodes.item(i).getNodeValue();
                        if (ns.validate(value)) {
                            ns.setLoadedValue(value);
                            ns.setValue(value);
                        }
                        //System.out.println("  " + ns.getName() + " = " + nodes.item(i).getNodeValue());
                    }
                } catch (XPathExpressionException xpeex) {
                    System.err.println("" + xpeex);
                    xpeex.printStackTrace();
                }
            }
            setLoadedAt(System.currentTimeMillis());
            return true;
        } catch (IOException ioex) {
            System.err.println("" + ioex);
        } catch (ParserConfigurationException pcex) {
            System.err.println("" + pcex);
        } catch (SAXException sex) {
            System.err.println("" + sex);
        }
        return false;
    }

    public boolean save(InputStream is, OutputStream os) {
        Document doc = null;
        try {
            DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
            domFactory.setNamespaceAware(false);
            DocumentBuilder builder = domFactory.newDocumentBuilder();
            //Document doc = builder.parse(zf.getInputStream(ze));
            doc = builder.parse(is);//sr.getInputStream());
            //zis.closeEntry();
            for (NSetting ns : getSettings()) {
                try {
                    XPath xpath = XPathFactory.newInstance().newXPath();
                    // XPath Query for showing all nodes value
                    XPathExpression expr = xpath.compile(ns.getLocator());
                    Object result = expr.evaluate(doc, XPathConstants.NODESET);
                    NodeList nodes = (NodeList) result;
                    for (int i = 0; i < nodes.getLength(); i++) {
                        if (ns.isModified() && !ns.isReadOnly()) {
                            nodes.item(i).setNodeValue(ns.getValue());
                            //System.out.println("  " + ns.getName() + " = " + nodes.item(i).getNodeValue());
                        }
                    }
                } catch (XPathExpressionException xpeex) {
                    System.err.println("" + xpeex);
                    xpeex.printStackTrace();
                }
            }
        } catch (IOException ioex) {
            System.err.println("" + ioex);
            ioex.printStackTrace();
        } catch (ParserConfigurationException pcex) {
            System.err.println("" + pcex);
            pcex.printStackTrace();
        } catch (SAXException sex) {
            System.err.println("" + sex);
            sex.printStackTrace();
        }

        if (doc != null) {
//            sr.prepareForWrite(true);
//            OutputStream os = sr.getOutputStream();

            // Write the DOM document to the file
            Source source = new DOMSource(doc);
            Result result = new StreamResult(os);
            try {
                Transformer xformer = TransformerFactory.newInstance().newTransformer();
                xformer.transform(source, result);
            } catch (TransformerConfigurationException tcex) {
                tcex.printStackTrace();
            } catch (TransformerException tex) {
                tex.printStackTrace();
            }
            //zos.closeEntry();
            // loaded value = value -> no modifications left.
            for (NSetting ns : getSettings()) {
                ns.setLoadedValue(ns.getValue());
            }
            setAppliedAt(System.currentTimeMillis());
            return true;
        } else {
            // copy input to output
            //sr.copyInputToOutput();
            return false;
        }
    }

    @Override
    public NSettingSource instanceFor(String path) {
        if (path != null) {
            return new XMLSettingSource(path);
        } else {
            return null;
        }
    }
}
