/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.data.types;

import ssg.tools.easyconfig.data.NSetting;
import ssg.tools.easyconfig.data.NValidationException;

/**
 *
 * @author ssg
 */
public class BinaryType extends BaseSettingType implements IBinarySettingType {

    public BinaryType(int size) {
        super(size);
    }

    public BinaryType() {
        super(0);
    }

    public String validate(NSetting ns, String value) throws NValidationException {
        // TODO: interpret string as byte[] ??
        return value;
    }

    public Class valueType() {
        return byte[].class;
    }

    public Object asValue(String value) {
        // TODO: interpret value as byte[] ??
        return value.getBytes();
    }

    public boolean supports(String type) {
        return "binary".equalsIgnoreCase(type);
    }

    public ISettingType instanceFor(String[] params) {
        int size = 255;
        if (params != null && params.length > 0) {
            try {
                size = NumericType.string2number(params[0], 0).intValue();
            } catch (Throwable th) {
            }
        }
        return new BinaryType(size);
    }

    ///////////////////////////////////////////////////////// IBinarySettingType
    public void setBinary(NSetting ns, byte[] data) throws NValidationException {
        if (data != null) {
            try {
                ns.setValue(new String(data, "ISO-8859-1"));
            } catch (Throwable th) {
            }
        } else {
            ns.setValue(null);
        }
    }

    public byte[] getBinary(NSetting ns) {
        if (ns.getValue() != null) {
            try {
                return ns.getValue().getBytes("ISO-8859-1");
            } catch (Throwable th) {
                return null;
            }
        } else {
            return null;
        }
    }

    public int getBinarySize() {
        return getSize();
    }
}
