/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.data.types;

import ssg.tools.easyconfig.data.NSetting;
import ssg.tools.easyconfig.data.NValidationException;
import java.io.File;

/**
 *
 * @author ssg
 */
public class FileType extends BaseSettingType {

    boolean asFolder = false;
    boolean checkExists = false;

    public FileType() {
    }

    public FileType(boolean asFolder) {
        this.asFolder = asFolder;
    }

    public FileType(boolean asFolder, boolean checkExists) {
        this.asFolder = asFolder;
        this.checkExists = checkExists;
    }

    public String validate(NSetting ns, String value) throws NValidationException {
        testMandatory(ns, value);
        if (value == null) {
            return null;
        }
        try {
            File f = new File(value);
            if (checkExists) {
                if (!f.exists()) {
                    throw new NValidationException("Specified " + value + " " + ((asFolder) ? "folder" : "file") + " is missing");
                }
                if (asFolder && !f.isDirectory()) {
                    throw new NValidationException("Not a folder: " + value);
                } else if (!asFolder && !f.isFile()) {
                    throw new NValidationException("Not a file: " + value);
                }
            }
            return value;
        } catch (NValidationException nvex) {
            throw nvex;
        } catch (Throwable th) {
            throw new NValidationException("Failed to validate '" + value + "' as " + ((asFolder) ? "folder" : "file") + ": " + th);
        }
    }

    public Class valueType() {
        return File.class;
    }

    public File asValue(String value) {
        if (value == null) {
            return null;
        }
        return new File(value);
    }

    public boolean supports(String type) {
        return "file".equalsIgnoreCase(type)
                || "folder".equalsIgnoreCase(type)
                || "directory".equalsIgnoreCase(type);
    }

    public ISettingType instanceFor(String[] params) {
        if (params == null || params.length < 1 || !supports(params[0])) {
            return null;
        }
        return new FileType(
                "folder".equalsIgnoreCase(params[0]) 
                || "directory".equalsIgnoreCase(params[0]));
    }
}
