/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.data.types;

import ssg.tools.easyconfig.data.NValidationException;
import ssg.tools.easyconfig.data.NSetting;

/**
 *
 * @author ssg
 */
public interface ISettingType {

    /**
     * Validates value for provided setting.
     * 
     * @param ns
     * @param value
     * @return
     * @throws NValidationException 
     */
    String validate(NSetting ns, String value) throws NValidationException;

    /**
     * Retruns java class representing the value handled in type setting
     * @return 
     */
    Class valueType();

    /**
     * Returns max string size to represent the value
     * 
     * @return 
     */
    int getSize();

    /**
     * Retruns value (object) representation for string
     * @param value
     * @return 
     */
    Object asValue(String value);

    /**
     * Returns true if type name is supported by this instance
     * 
     * @param type
     * @return 
     */
    boolean supports(String type);

    /**
     * Creates instance for provided parameters.
     * 1st string is type name (must be supported)
     * Other values are as expected by type implementation, usually name=value pairs.
     * 
     * @param params
     * @return 
     */
    ISettingType instanceFor(String[] params);
}
