/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.data.types;

import ssg.tools.easyconfig.utils.NFactoryPrototype;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.util.ServiceLoader;

/**
 *
 * @author ssg
 */
public class SettingTypeFactory extends NFactoryPrototype<ISettingType> {

    private static ServiceLoader<ISettingType> typesLoader = ServiceLoader.load(ISettingType.class);
    static SettingTypeFactory factory;

    protected SettingTypeFactory() {
    }

    public static SettingTypeFactory getInstance() {
        if (factory == null) {
            factory = new SettingTypeFactory();
            factory.initDefaultTypes(true);
        }
        return factory;
    }

    public void loadTypeDefinitions(File typeDefinitionsFile) throws IOException {
        loadTypeDefinitions(new FileInputStream(typeDefinitionsFile));
    }

    public void loadTypeDefinitions(InputStream is) throws IOException {
        LineNumberReader lnr = new LineNumberReader(new InputStreamReader(new BufferedInputStream(is), "UTF-8"));
        String s = null;
        while ((s = lnr.readLine()) != null) {
            s = s.trim();
            if (s.startsWith("#")) {
                continue;
            }
            int idxComment = s.indexOf("#");
            int idx = s.indexOf("=");
            if (idx == -1 || idxComment >= 0 && idx > idxComment) {
                continue;
            }
            String name = s.substring(0, idx);
            s = s.substring(idx + 1);
            if (!types.containsKey(name)) {
                // class or class alias, class constructor parameters as name=value
                String[] ss = s.split(";");
                String tt = ss[0];
                ISettingType nt = null;

                for (ISettingType st : typesLoader) {
                    if (st != null && st.supports(tt)) {
                        nt = st.instanceFor(ss);
                    }
                }
                if (nt != null) {
                    types.put(name, nt);
                } else {
                    System.err.println(getClass().getName() + ".loadTypeDefinitions: unsupported type '" + name + "' with parameters '" + s + "'");
                }
            }
        }
        lnr.close();
    }

    public void saveTypeDefinitions(File typeDefinitionsFile) throws IOException {
    }

    @Override
    public boolean matchType(ISettingType type, Object obj, boolean exact) {
        if (type == null || obj == null) {
            return false;
        }
        Class oc = obj.getClass();
        return (type.valueType() != null && (exact && type.valueType().equals(oc) || type.valueType().isAssignableFrom(oc)));
    }
}
