/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.data.types;

import ssg.tools.easyconfig.data.NSetting;
import ssg.tools.easyconfig.data.NValidationException;

/**
 *
 * @author ssg
 */
public class StringType extends BaseSettingType {

    int minLen;
    String mask;
    String regExp;
    private String encoding;

    public StringType() {
        super(255);
    }

    public StringType(int maxLen) {
        super(maxLen);
    }

    public StringType(int minLen, int maxLen, String mask, String regExp) {
        super(maxLen);
        this.minLen = minLen;
        this.mask = mask;
        this.regExp = regExp;
    }

    public String validate(NSetting ns, String value) throws NValidationException {
        testMandatory(ns, value);
        return value;
    }

    public Class valueType() {
        return String.class;
    }

    public String asValue(String value) {
        return value;
    }

    public boolean supports(String type) {
        return "string".equalsIgnoreCase(type) || "text".equalsIgnoreCase(type);
    }

    public ISettingType instanceFor(String[] params) {
        if (params == null || params.length < 1 || !supports(params[0])) {
            return null;
        }
        return new StringType();
    }

    /**
     * @return the encoding
     */
    public String getEncoding() {
        return encoding;
    }

    /**
     * @param encoding the encoding to set
     */
    public void setEncoding(String encoding) {
        this.encoding = encoding;
    }
}
