/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.data.types;

import ssg.tools.easyconfig.data.NSetting;
import ssg.tools.easyconfig.data.NValidationException;
import java.net.URI;
import java.net.URISyntaxException;

/**
 *
 * @author ssg
 */
public class UriType extends BaseSettingType {

    public String validate(NSetting ns, String value) throws NValidationException {
        testMandatory(ns, value);
        if (value == null) {
            return null;
        }
        try {
            URI uri = new URI(value);
        } catch (URISyntaxException usex) {
            throw new NValidationException("Invalid URL '" + value + "': " + usex);
        }
        return value;
    }

    public Class valueType() {
        return URI.class;
    }

    public URI asValue(String value) {
        if (value == null) {
            return null;
        }
        try {
            return new URI(value);
        } catch (URISyntaxException usex) {
            return null;
        }
    }

    public boolean supports(String type) {
        return "uri".equalsIgnoreCase(type);
    }

    public ISettingType instanceFor(String[] params) {
        if (params == null || params.length < 1 || !supports(params[0])) {
            return null;
        }
        return new UriType();
    }
}
