/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.models;

import ssg.tools.easyconfig.data.handler.IConfigurationHandler;
import ssg.tools.easyconfig.data.NSetting;
import ssg.tools.easyconfig.data.NSettingSource;
import java.util.HashMap;
import java.util.Map;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

/**
 *
 * @author ssg
 */
public class ConfigurationTreeModel extends DefaultTreeModel {

    public static final int MODE_UNKNOWN = -1;
    public static final int MODE_SOURCES = 0;
    public static final int MODE_GROUPS = 1;
    Map<Object, DefaultMutableTreeNode> cache = new HashMap<Object, DefaultMutableTreeNode>();
    IConfigurationHandler config;
    private int mode = MODE_SOURCES;

    public ConfigurationTreeModel(IConfigurationHandler config) {
        super(new DefaultMutableTreeNode(config));
        this.config = config;
        cache.put(config, (DefaultMutableTreeNode) this.getRoot());
    }

//    public ConfigurationTreeModel(TreeNode root, boolean asksAllowsChildren) {
//        super(root, asksAllowsChildren);
//    }
//
//    public ConfigurationTreeModel(TreeNode root) {
//        super(root);
//    }
    @Override
    public Object getChild(Object parent, int index) {
        Object obj = getUserObject(parent);
        if (obj instanceof IConfigurationHandler) {
            switch (getMode()) {
                case MODE_SOURCES:
                    obj = ((IConfigurationHandler) obj).getSettings().get(index);
                    break;
                case MODE_GROUPS:
                    obj = ((IConfigurationHandler) obj).getGroups().get(index);
                    break;
                default:
            }
        } else if (obj instanceof NSettingSource) {
            obj = ((NSettingSource) obj).getSettings().get(index);
        } else if (obj instanceof String && getMode() == MODE_GROUPS) {
            obj = config.getGroup((String) obj).get(index);
        } else {
            return null;
        }
        if (cache.containsKey(obj)) {
            return cache.get(obj);
        } else {
            DefaultMutableTreeNode tn = new DefaultMutableTreeNode(obj);
            cache.put(obj, tn);
            return tn;
        }
    }

    @Override
    public int getChildCount(Object parent) {
        Object obj = getUserObject(parent);
        if (obj instanceof IConfigurationHandler) {
            switch (getMode()) {
                case MODE_SOURCES:
                    return ((IConfigurationHandler) obj).getSettings().size();
                case MODE_GROUPS:
                    return ((IConfigurationHandler) obj).getGroups().size();
                default:
                    return 0;
            }
        } else if (obj instanceof NSettingSource) {
            return ((NSettingSource) obj).getSettings().size();
        } else if (obj instanceof String && getMode() == MODE_GROUPS) {
            return config.getGroup((String) obj).size();
        } else {
            return -1;
        }
    }

    @Override
    public int getIndexOfChild(Object parent, Object child) {
        Object obj = getUserObject(parent);
        if (obj instanceof IConfigurationHandler) {
            return ((IConfigurationHandler) obj).getSettings().indexOf(child);
        } else if (obj instanceof NSettingSource) {
            return ((NSettingSource) obj).getSettings().indexOf(child);
        } else if (obj instanceof String && getMode() == MODE_GROUPS) {
            return config.getGroup((String) obj).indexOf(child);
        } else {
            return -1;
        }
    }

    public void clearCache(Object obj) {
        if (obj != null) {
            obj = getUserObject(obj);
            if (obj != null) {
                cache.remove(obj);
            }
        } else {
            cache.clear();
        }
    }

    @Override
    public boolean isLeaf(Object node) {
        return getSetting(node) != null;
    }

    public static Object getUserObject(Object obj) {
        if (obj instanceof TreePath) {
            obj = ((TreePath) obj).getLastPathComponent();
        }
        if (obj instanceof DefaultMutableTreeNode) {
            return ((DefaultMutableTreeNode) obj).getUserObject();
        } else {
            return obj;
        }
    }

    NSettingSource getSettingSource(Object obj) {
        obj = getUserObject(obj);
        if (obj instanceof NSettingSource) {
            return (NSettingSource) obj;
        } else {
            return null;
        }
    }

    NSetting getSetting(Object obj) {
        obj = getUserObject(obj);
        if (obj instanceof NSetting) {
            return (NSetting) obj;
        } else {
            return null;
        }
    }

    /**
     * @return the mode
     */
    public int getMode() {
        return mode;
    }

    /**
     * @param mode the mode to set
     */
    public void setMode(int mode) {
        this.mode = mode;
    }

    public void switchMode(int nextMode) {
        if (nextMode != MODE_UNKNOWN) {
            mode = nextMode;
        } else if (mode == MODE_SOURCES) {
            mode = MODE_GROUPS;
        } else {
            mode = MODE_SOURCES;
        }
        clearCache(null);
        //nodeChanged((TreeNode) getRoot());
        this.fireTreeStructureChanged(getRoot(), (new TreePath(getRoot())).getPath(), null, null);
    }
}
