/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.models;

import ssg.tools.easyconfig.data.NSetting;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;
import javax.swing.table.DefaultTableModel;

/**
 *
 * @author ssg
 */
public class NSettingTableModel extends DefaultTableModel {

    List<NSetting> settings;

    public NSettingTableModel(List<NSetting> settings, boolean allColumns) {
        super(toDataVector(settings), toColumnsVector(settings,allColumns));
        this.settings = settings;
    }

    public void setSettings(List<NSetting> settings, boolean allColumns) {
        this.settings = settings;
        this.setDataVector(toDataVector(settings), toColumnsVector(settings, allColumns));
        this.fireTableDataChanged();
    }

    /**
     * Returns COPY of internally stored list only.
     *
     * @return
     */
    public List<NSetting> getSettings() {
        ArrayList<NSetting> result = new ArrayList<NSetting>();
        if (settings != null) {
            result.addAll(settings);
        }
        return result;
    }

    public NSetting getSetting(int row) {
        if (settings != null) {
            if (row >= 0 && row < settings.size()) {
                return settings.get(row);
            }
        }
        return null;
    }

    @Override
    public void setValueAt(Object aValue, int row, int column) {
        if (column == 1) {
            NSetting ns = settings.get(row);
            if (ns.validate("" + aValue)) {
                ns.setValue("" + aValue);
                super.setValueAt(aValue, row, column);
            } else {
                // report or indicate save error for table...
            }
        }
    }

    @Override
    public boolean isCellEditable(int row, int column) {
        return column == 1 && !settings.get(row).isReadOnly();
    }

    public boolean IsModifiedCell(int row, int column) {
        return column == 1 && settings.get(row).isModified();

    }

    static Vector toDataVector(List<NSetting> settings) {
        Vector v = new Vector();
        if (settings != null) {
            for (NSetting ns : settings) {
                Vector vs = new Vector();
                vs.add(ns.getName());
                vs.add(ns.getValue());
                vs.add(ns.getLoadedValue());
                vs.add(ns.getDefaultValue());
                vs.add(ns.getLocator());
                vs.add(ns.getSource().getPath());
                v.add(vs);
            }
        }
        return v;
    }

    static Vector toColumnsVector(List<NSetting> settings, boolean allColumns) {
        Vector v = new Vector();
        v.add("Name");
        v.add("value");
        if(allColumns) {
            v.add("loaded");
            v.add("default");
            v.add("locator");
            v.add("source");
        }
        return v;
    }
}
