/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.utils.tar;

import java.io.IOException;
import java.io.InputStream;

/**
 *
 * @author ssg
 */
public class TarInputStream extends InputStream {
    InputStream is;
    TarEntryHeader entry;
    InputStream entryStream;

    public TarInputStream(InputStream is) {
        this.is = is;
    }

    @Override
    public int read() throws IOException {
        return (entryStream != null) ? entryStream.read() : -1;
    }

    public TarEntryHeader nextEntry() throws IOException {
        if (entry != null) {
            closeEntry();
        }
        entry = new TarEntryHeader(is);
        if (entry.getSize() != -1) {
            entryStream = new NestedTarInputStream(is, entry.getSize());
        }
        return entry;
    }

    public void closeEntry() throws IOException {
        if (entryStream != null && entryStream.available() > 0) {
            while (entryStream.read() != -1) {
                ;
            }
            entryStream.close();
        }
        entry = null;
        entryStream = null;
    }

    public void close() throws IOException {
        closeEntry();
        is.close();
        is = null;
    }
    
}
