/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.utils.tar;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 *
 * @author ssg
 */
public class TarRecord {
    public static final int RECORDSIZE = 512;
    byte[] data = new byte[RECORDSIZE];
    int length;

    public TarRecord(InputStream is) throws IOException {
        //length = is.read(data);
        loadData(is, data.length);
    }

    public TarRecord(InputStream is, long sizeLeft) throws IOException {
        //length = is.read(data, 0, (sizeLeft > RECORDSIZE) ? RECORDSIZE : (int) sizeLeft);
        loadData(is, (sizeLeft > RECORDSIZE) ? RECORDSIZE : (int) sizeLeft);
    }

    void loadData(InputStream is, int size) throws IOException {
        int cnt = is.read(data);
        int retries = 3;
        while (cnt < size && cnt > 0) {
            int lc = is.read(data, cnt, size - cnt);
            if (lc < 0) {
                retries--;
                if (retries == 0) {
                    break;
                }
                //throw new IOException("EOF detected: tar data block loading is unfinished[" + cnt + " out of "+size+" bytes]: " + this.toString());
            }
            cnt += lc;
        }
        length = cnt;
    }

    public int getLength() {
        return length;
    }

    public TarEntryHeader toTarEntryHeader() {
        try {
            return new TarEntryHeader(new ByteArrayInputStream(data));
        } catch (IOException ioex) {
            return null;
        }
    }
    
}
