/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.vc;

import ssg.tools.easyconfig.data.NSettingSource;
import ssg.tools.easyconfig.data.handler.CompositeConfigurationHandler;
import ssg.tools.easyconfig.data.handler.IConfigurationHandler;
import ssg.tools.easyconfig.data.handler.TemplateConfigurationHandler;
import ssg.tools.easyconfig.models.ConfigurationChangeEvent;
import ssg.tools.easyconfig.models.ConfigurationChangeListener;
import ssg.tools.easyconfig.models.ConfigurationTreeModel;
import ssg.tools.easyconfig.utils.IOUtils;
import ssg.tools.easyconfig.vc.behaviours.ChangeConfigModeTreeMouseListener;
import ssg.tools.easyconfig.vc.behaviours.TreeExpanderKeyListener;
import java.awt.Color;
import java.awt.Component;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Vector;
import javax.swing.JComponent;
import javax.swing.TransferHandler;
import javax.swing.border.TitledBorder;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

/**
 *
 * @author ssg
 */
public class ConfiguratorComponentsPanel
        extends javax.swing.JPanel
        implements IConfiguratorPanel, ConfigurationChangeListener {

    /**
     * Creates new form ConfiguratorComponentsPanel
     */
    public ConfiguratorComponentsPanel() {
        initComponents();
        initCustom();
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabelTemplate = new javax.swing.JLabel();
        jLabelSource = new javax.swing.JLabel();
        jLabelTarget = new javax.swing.JLabel();
        jPanelDetails = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jListSources = new javax.swing.JList();
        jScrollPane2 = new javax.swing.JScrollPane();
        jTreeSettings = new javax.swing.JTree();
        jScrollPane3 = new javax.swing.JScrollPane();
        jListOptions = new javax.swing.JList();
        jButtonClearTemplate = new javax.swing.JButton();
        jButtonSaveTemplate = new javax.swing.JButton();
        jButtonAdjustTemplate = new javax.swing.JButton();

        jLabelTemplate.setText("empty template");
        jLabelTemplate.setBorder(javax.swing.BorderFactory.createTitledBorder("Template"));
        jLabelTemplate.setFocusable(false);

        jLabelSource.setText("no source");
        jLabelSource.setBorder(javax.swing.BorderFactory.createTitledBorder("Source"));

        jLabelTarget.setText("no explicit target");
        jLabelTarget.setBorder(javax.swing.BorderFactory.createTitledBorder("Target"));

        jPanelDetails.setBorder(javax.swing.BorderFactory.createTitledBorder("Details"));
        jPanelDetails.setOpaque(false);

        jListSources.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        jScrollPane1.setViewportView(jListSources);

        jScrollPane2.setViewportView(jTreeSettings);

        jListOptions.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        jScrollPane3.setViewportView(jListOptions);

        javax.swing.GroupLayout jPanelDetailsLayout = new javax.swing.GroupLayout(jPanelDetails);
        jPanelDetails.setLayout(jPanelDetailsLayout);
        jPanelDetailsLayout.setHorizontalGroup(
            jPanelDetailsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1)
            .addComponent(jScrollPane2)
            .addComponent(jScrollPane3)
        );
        jPanelDetailsLayout.setVerticalGroup(
            jPanelDetailsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanelDetailsLayout.createSequentialGroup()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 121, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, 0, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, 111, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        jButtonClearTemplate.setText("Clear template");
        jButtonClearTemplate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonClearTemplateActionPerformed(evt);
            }
        });

        jButtonSaveTemplate.setText("Save template");

        jButtonAdjustTemplate.setText("Adjust template");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(2, 2, 2)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jButtonClearTemplate)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jButtonSaveTemplate)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jButtonAdjustTemplate)
                        .addContainerGap(108, Short.MAX_VALUE))
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                            .addComponent(jLabelSource, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(jLabelTarget, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 218, Short.MAX_VALUE)
                            .addComponent(jLabelTemplate, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jPanelDetails, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jLabelTemplate, javax.swing.GroupLayout.DEFAULT_SIZE, 125, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabelSource, javax.swing.GroupLayout.DEFAULT_SIZE, 125, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabelTarget, javax.swing.GroupLayout.PREFERRED_SIZE, 120, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(jPanelDetails, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addGap(5, 5, 5)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButtonClearTemplate)
                    .addComponent(jButtonSaveTemplate)
                    .addComponent(jButtonAdjustTemplate)))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void jButtonClearTemplateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonClearTemplateActionPerformed
        // TODO add your handling code here:
        if (template != null) {
            template.getTemplates().clear();
            template.getOptions().clear();
            template.setSettings(new ArrayList<NSettingSource>(1), true);
            template.setLastLoaded(System.currentTimeMillis());
            updateTemplate();
            setConfig(getConfig());
        }
    }//GEN-LAST:event_jButtonClearTemplateActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButtonAdjustTemplate;
    private javax.swing.JButton jButtonClearTemplate;
    private javax.swing.JButton jButtonSaveTemplate;
    private javax.swing.JLabel jLabelSource;
    private javax.swing.JLabel jLabelTarget;
    private javax.swing.JLabel jLabelTemplate;
    private javax.swing.JList jListOptions;
    private javax.swing.JList jListSources;
    private javax.swing.JPanel jPanelDetails;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JTree jTreeSettings;
    // End of variables declaration//GEN-END:variables
    private TemplateConfigurationHandler template;
    private IConfigurationHandler source;
    private IConfigurationHandler target;
    private IConfigurationHandler detailsCH;
    List<ConfigurationChangeListener> listeners = new ArrayList<ConfigurationChangeListener>();

    public void setConfig(IConfigurationHandler config) {
        source = config;
        if (source != null && template != null) {
            if (source.getLastLoaded() < template.getLastModified()) {
                try {
                    source.setSettings(template.cloneSettings(), true);
                    source.loadSettings();
                } catch (IOException ioex) {
                    ioex.printStackTrace();
                }
            }
        }
        updateSource();
    }

    public IConfigurationHandler getConfig() {
        return source;
    }

    /**
     * @return the template
     */
    public TemplateConfigurationHandler getTemplate() {
        return template;
    }

    /**
     * @param template the template to set
     */
    public void setTemplate(TemplateConfigurationHandler template) {
        this.template = template;
        updateTemplate();
        if (getConfig() != null) {
            setConfig(getConfig());
        }
    }

    /**
     * @return the target
     */
    public IConfigurationHandler getTarget() {
        return target;
    }

    /**
     * @param target the target to set
     */
    public void setTarget(IConfigurationHandler target) {
        this.target = target;
        updateTarget();
    }

    public void updateSource() {
        if (source != null) {
            StringBuilder sb = new StringBuilder();
            sb.append("<html><b>");
            try {
                File f = new File(source.getUri());
                sb.append(f.getName());
            } catch (Throwable th) {
                sb.append(source.getUri().getPath());
            }
            sb.append("</b></html>");
            jLabelSource.setText(sb.toString());
            jLabelSource.setToolTipText(source.getUri().getPath());
        } else {
            jLabelSource.setText("no source");
        }
        this.fireConfigurationChange(new ConfigurationChangeEvent(this, null, source, null));
    }

    public void updateTemplate() {
        if (template != null) {
            StringBuilder sb = new StringBuilder();
            sb.append("<html><b>");
            sb.append((template.getUri() != null) ? template.getUri().getPath() : "no path");
            sb.append("<br/>Setting sources: "+template.getSettings().size());
            for (URI u : template.getTemplates()) {
                try {
                    File f = new File(u);
                    sb.append("<li>" + f.getName());
                } catch (Throwable th) {
                    sb.append("<li>" + u.getPath());
                }
            }
            sb.append("</b></html>");
            jLabelTemplate.setText(sb.toString());
            jLabelTemplate.setToolTipText(template.toString());
        } else {
            jLabelTemplate.setText("no template");
        }
        this.fireConfigurationChange(new ConfigurationChangeEvent(this, template, null, null));
    }

    public void updateTarget() {
        this.fireConfigurationChange(new ConfigurationChangeEvent(this, null, null, target));
    }

    void initCustom() {
        this.setTransferHandler(new TransferHandler() {
            List<String> dfns = new ArrayList<String>() {
                {
                    add("application/x-java-url");
                    add("text/plain");
                    add("application/x-java-file-list");
                }
            };

            @Override
            public boolean importData(TransferHandler.TransferSupport support) {
                System.out.println("importData(" + support + ")");
                for (DataFlavor df : support.getDataFlavors()) {
                    String dfn = df.getHumanPresentableName();
                    System.out.print("import DF: " + dfn + ": ");
                    try {
                        System.out.println(support.getTransferable().getTransferData(df));
                    } catch (Throwable th) {
                        System.out.println("ERROR: " + th);
                    }

                    if (dfns.contains(dfn)) {
                        try {
                            Object o = support.getTransferable().getTransferData(df);
                            if (o instanceof String) {
                                System.out.println("import: " + df.getHumanPresentableName());
                                System.out.println("... [" + o.getClass() + "]" + ((o instanceof String) ? o : "<>"));
                                doImportData(ConfiguratorComponentsPanel.this.getComponentAt(support.getDropLocation().getDropPoint()), o);
                            }
                            if (o instanceof List) {
                                for (Object oo : (List) o) {
                                    System.out.println("import(" + ((List) o).indexOf(oo) + "): " + df.getHumanPresentableName());
                                    if (oo instanceof String || oo instanceof URL || oo instanceof File) {
                                        System.out.println("... ... [" + oo.getClass() + "]" + oo);
                                        doImportData(ConfiguratorComponentsPanel.this.getComponentAt(support.getDropLocation().getDropPoint()), oo);
                                    }
                                }
                            }
                        } catch (UnsupportedFlavorException ufex) {
                        } catch (IOException ioex) {
                        }
                    }
                }
                return true;
            }

            @Override
            public boolean importData(JComponent comp, Transferable t) {
                System.out.println("importData(" + comp.getClass().getSimpleName() + ", " + t + ")");
                for (DataFlavor df : t.getTransferDataFlavors()) {
                    System.out.println("import: " + df.getHumanPresentableName());
                }
                return true;
            }

            @Override
            public boolean canImport(TransferHandler.TransferSupport support) {
                //System.out.println("canImport(" + support + ")");
                for (DataFlavor df : support.getDataFlavors()) {
                    //System.out.println("canImport: " + df.getHumanPresentableName());
                }
                return support.isDrop();
            }

            @Override
            public boolean canImport(JComponent comp, DataFlavor[] transferFlavors) {
                //System.out.println("canImport(" + comp.getClass().getSimpleName() + ", " + transferFlavors + ")");
                for (DataFlavor df : transferFlavors) {
                    //System.out.println("canImport: " + df.getHumanPresentableName());
                }
                return true;
            }
        });

        jLabelSource.addMouseListener(new MouseListener() {
            public void mouseClicked(MouseEvent e) {
            }

            public void mousePressed(MouseEvent e) {
            }

            public void mouseReleased(MouseEvent e) {
            }

            public void mouseEntered(MouseEvent e) {
                setDetailsCH(source);
            }

            public void mouseExited(MouseEvent e) {
            }
        });
        jLabelTemplate.addMouseListener(new MouseListener() {
            public void mouseClicked(MouseEvent e) {
            }

            public void mousePressed(MouseEvent e) {
            }

            public void mouseReleased(MouseEvent e) {
            }

            public void mouseEntered(MouseEvent e) {
                setDetailsCH(template);
            }

            public void mouseExited(MouseEvent e) {
            }
        });
        jLabelTarget.addMouseListener(new MouseListener() {
            public void mouseClicked(MouseEvent e) {
            }

            public void mousePressed(MouseEvent e) {
            }

            public void mouseReleased(MouseEvent e) {
            }

            public void mouseEntered(MouseEvent e) {
                setDetailsCH(target);
            }

            public void mouseExited(MouseEvent e) {
            }
        });

        jTreeSettings.addMouseListener(new ChangeConfigModeTreeMouseListener());
        jTreeSettings.addKeyListener(new TreeExpanderKeyListener());
        setDetailsCH(null);
    }

    void doImportData(Component c, Object obj) throws IOException {
        if (c == null || obj == null) {
            return;
        }
        URI u = null;
        try {
            if (obj instanceof File) {
                u = ((File) obj).toURI();
            } else if (obj instanceof URL) {
                u = ((URL) obj).toURI();
            } else if (obj instanceof URI) {
                u = (URI) obj;
            } else {
                u = new URI(obj.toString());
            }
        } catch (URISyntaxException usex) {
        }
        if (u == null) {
            return;
        }
        if (jLabelTemplate.equals(c)) {
            // import as template or template definition?
            try {
                if (u.getPath().endsWith(template.TEMPLATE_EXTENSION)) {
                    // load as template definition file
                    template.setUri(u);
                    template.loadSettings();
                } else {
                    List<NSettingSource> tmp = template.loadConfigurationTemplate(IOUtils.uri2InputStream(u, null), "*");
                    template.setSettings(tmp, false);
                    template.getTemplates().add(u);
                }
                setTemplate(getTemplate());
            } catch (Throwable th) {
                th.printStackTrace();
            }
            setDetailsCH(null);
            setDetailsCH(template);
        } else if (jLabelSource.equals(c)) {
            setConfig(new CompositeConfigurationHandler(u));
            //updateSource();
            setDetailsCH(source);
        }
    }

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D) g.create();
        Color c1 = Color.white;
        float[] hsb = new float[3];
        int grayLevel = 212;
        Color.RGBtoHSB(grayLevel, grayLevel, grayLevel, hsb);
        Color c2 = Color.getHSBColor(hsb[0], hsb[1], hsb[2]);
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        GradientPaint gp = new GradientPaint(0, 0,
                c1,
                getWidth() * 2 / 3, getHeight(),
                c2);
        g2.setPaint(gp);
        g2.fillRect(0, 0, getWidth(), getHeight());
        g2.drawString("Creative split pane", 10, getHeight() / 2 + 20 / 2);
    }

    public void addConfigurationChangeListener(ConfigurationChangeListener l) {
        if (l != null && !listeners.contains(l)) {
            listeners.add(l);
        }
    }

    public void removeConfigurationChangeListener(ConfigurationChangeListener l) {
        if (l != null && listeners.contains(l)) {
            listeners.remove(l);
        }
    }

    /**
     * fire changes with loop avoidance logic...
     *
     * @param e
     */
    public void fireConfigurationChange(ConfigurationChangeEvent e) {
        if (isLockedFire(e)) {
            return;
        }
        lockFire(e, true);
        try {
            for (ConfigurationChangeListener l : listeners) {
                l.configurationChanged(e);
            }
        } finally {
            lockFire(e, false);
        }
    }

    synchronized boolean isLockedFire(ConfigurationChangeEvent e) {
        if (e == null) {
            return false;
        }
        if (e.getChangedSetting() != null && lockedFire.contains(e.getChangedSetting())) {
            return true;
        }
        if (e.getChangedSettingSource() != null && lockedFire.contains(e.getChangedSettingSource())) {
            return true;
        }
        if (e.getChangedSource() != null && lockedFire.contains(e.getChangedSource())) {
            return true;
        }
        if (e.getChangedTarget() != null && lockedFire.contains(e.getChangedTarget())) {
            return true;
        }
        if (e.getChangedTemplate() != null && lockedFire.contains(e.getChangedTemplate())) {
            return true;
        }
        return false;
    }

    synchronized void lockFire(ConfigurationChangeEvent e, boolean lock) {
        if (e == null) {
            return;
        }
        Object[] objs = new Object[5];
        objs[0] = e.getChangedSetting();
        objs[1] = e.getChangedSettingSource();
        objs[2] = e.getChangedSource();
        objs[3] = e.getChangedTarget();
        objs[4] = e.getChangedTemplate();
        for (Object obj : objs) {
            if (obj != null) {
                if (!lockedFire.contains(obj) && lock) {
                    lockedFire.add(obj);
                } else if (lockedFire.contains(obj) && !lock) {
                    lockedFire.remove(obj);
                }
            }
        }
    }
    private Set lockedFire = new HashSet();

    /**
     * @return the detailsCH
     */
    public IConfigurationHandler getDetailsCH() {
        return detailsCH;
    }

    /**
     * @param detailsCH the detailsCH to set
     */
    public void setDetailsCH(IConfigurationHandler detailsCH) {
        //System.out.println("Set details for " + detailsCH);
        if (detailsCH != null && !detailsCH.equals(this.detailsCH)) {
            this.detailsCH = detailsCH;
            this.jListSources.setListData(new Vector(detailsCH.getSettings()));
            if (detailsCH instanceof TemplateConfigurationHandler) {
                this.jListOptions.setListData(((TemplateConfigurationHandler) detailsCH).getTemplates().toArray());
            } else if (detailsCH instanceof CompositeConfigurationHandler) {
                this.jListOptions.setListData(((CompositeConfigurationHandler) detailsCH).getSourceHandlers().toArray());
            } else {
                this.jListOptions.setListData(new Object[0]);
            }
            this.jTreeSettings.setModel(new ConfigurationTreeModel(detailsCH));

            TitledBorder tb = (TitledBorder) this.jPanelDetails.getBorder();
            URI u = detailsCH.getUri();
            if (u != null) {
                try {
                    File f = new File(u);
                    tb.setTitle("Details for " + f.getName());
                } catch (Throwable th) {
                    tb.setTitle("Details for " + u.getPath());
                }
            } else {
                tb.setTitle("Details for " + detailsCH);
            }
            this.jPanelDetails.repaint();
        } else if (detailsCH == null) {
            this.detailsCH = null;
            this.jListSources.setListData(new Object[0]);
            this.jListOptions.setListData(new Object[0]);
            this.jTreeSettings.setModel(new DefaultTreeModel(new DefaultMutableTreeNode("no data")) {
                public Object getRoot() {
                    return "no data";
                }

                public Object getChild(Object parent, int index) {
                    return null;
                }

                public int getChildCount(Object parent) {
                    return 0;
                }

                public boolean isLeaf(Object node) {
                    return true;
                }

                public void valueForPathChanged(TreePath path, Object newValue) {
                }

                public int getIndexOfChild(Object parent, Object child) {
                    return -1;
                }
            });

            TitledBorder tb = (TitledBorder) this.jPanelDetails.getBorder();
            tb.setTitle("Move mouse over item to see details here");
            this.jPanelDetails.repaint();
        }
    }

    public void configurationChanged(ConfigurationChangeEvent e) {
        // TODO configuration changed handling
    }
}
