/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.vc;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import javax.swing.ImageIcon;

/**
 *
 * @author ssg
 */
public class CreativePanel extends javax.swing.JPanel {

    private ImageIcon bgImage;
    private BG_POSITION bgPosition = BG_POSITION.stretch;
    private float bgAlpha = 0.4f;
    private Color gradientFromColor;
    private Color gradientToColor;
    private short gradientFromGrayLevel = 255;
    private short gradientToGrayLevel = 212;

    /**
     * Creates new form CreativePanel
     */
    public CreativePanel() {
        initComponents();
        this.setOpaque(false);
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 593, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 414, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables

    @Override
    protected void paintComponent(Graphics g) {
        drawBG(g);
        super.paintComponent(g);
        //super.paintComponent(g);
    }

    @Override
    protected void paintChildren(Graphics g) {
        super.paintChildren(g);
    }

    public void drawBG(Graphics g) {
        Graphics2D g2 = (Graphics2D) g.create();
        Color c1 = getGradientFromColor();
        Color c2 = getGradientToColor();
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        GradientPaint gp = new GradientPaint(0, 0,
                c1,
                getWidth() * 2 / 3, getHeight(),
                c2);
        g2.setPaint(gp);
        g2.fillRect(0, 0, getWidth(), getHeight());

        //g2.setColor(Color.red);
        //g2.drawString("Creative panel", 10, getHeight() / 2 + 20 / 2);
        if (getBgImage() != null && !BG_POSITION.none.equals(bgPosition)) {
            int x = 0;
            int y = 0;

            x = getWidth() / 2 - getBgImage().getIconWidth() / 2;
            if (x < 0) {
                x = 0;
            }
            switch (bgPosition) {
                case center: {
                    y = getHeight() / 2 - getBgImage().getIconHeight() / 2;
                    if (y < 0) {
                        y = 0;
                    }
                }
                break;
                case top:
                    y = 0;
                    break;
                case bottom:
                    y = getHeight() - getBgImage().getIconHeight();
                    if (y < 0) {
                        y = 0;
                    }
                    break;
                case stretch: {
                    x = 0;
                    y = 0;
                    double sx = 1.0 * getWidth() / getBgImage().getIconWidth();
                    double sy = 1.0 * getHeight() / getBgImage().getIconHeight();
                    g2.scale(sx, sy);
                }
                break;
            }
            g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getBgAlpha()));

            g2.drawImage(getBgImage().getImage(), x, y, getBgImage().getIconWidth(), getBgImage().getIconHeight(), this);
        }
    }

    /**
     * @return the bgImage
     */
    public ImageIcon getBgImage() {
        return bgImage;
    }

    /**
     * @param bgImage the bgImage to set
     */
    public void setBgImage(ImageIcon bgImage, BG_POSITION pos) {
        this.bgImage = bgImage;
        if (pos != null) {
            bgPosition = pos;
        }
    }

    /**
     * @return the bgAlpha
     */
    public float getBgAlpha() {
        return bgAlpha;
    }

    /**
     * @param bgAlpha the bgAlpha to set
     */
    public void setBgAlpha(float bgAlpha) {
        this.bgAlpha = bgAlpha;
    }

    /**
     * @return the gradientFromColor
     */
    public Color getGradientFromColor() {
        return (gradientFromColor != null) ? gradientFromColor : getGrayColor(getGradientFromGrayLevel());
    }

    /**
     * @param gradientFromColor the gradientFromColor to set
     */
    public void setGradientFromColor(Color gradientFromColor) {
        this.gradientFromColor = gradientFromColor;
    }

    /**
     * @return the gradientToColor
     */
    public Color getGradientToColor() {
        return (gradientToColor != null) ? gradientToColor : getGrayColor(getGradientToGrayLevel());
    }

    /**
     * @param gradientToColor the gradientToColor to set
     */
    public void setGradientToColor(Color gradientToColor) {
        this.gradientToColor = gradientToColor;
    }

    /**
     * @return the gradientFromGrayLevel
     */
    public short getGradientFromGrayLevel() {
        return gradientFromGrayLevel;
    }

    /**
     * @param gradientFromGrayLevel the gradientFromGrayLevel to set
     */
    public void setGradientFromGrayLevel(short gradientFromGrayLevel) {
        this.gradientFromGrayLevel = gradientFromGrayLevel;
    }

    /**
     * @return the gradientToGrayLevel
     */
    public short getGradientToGrayLevel() {
        return gradientToGrayLevel;
    }

    /**
     * @param gradientToGrayLevel the gradientToGrayLevel to set
     */
    public void setGradientToGrayLevel(short gradientToGrayLevel) {
        this.gradientToGrayLevel = gradientToGrayLevel;
    }

    public Color getGrayColor(int grayLevel) {
        float[] hsb = new float[3];
        Color.RGBtoHSB(grayLevel, grayLevel, grayLevel, hsb);
        return Color.getHSBColor(hsb[0], hsb[1], hsb[2]);
    }
}
