/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.vc;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Component;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import javax.swing.ImageIcon;
import javax.swing.JSplitPane;

/**
 *
 * @author ssg
 */
public class CreativeSplitPane extends JSplitPane {
    private ImageIcon bgImage;
    private BG_POSITION bgPosition = BG_POSITION.stretch;

    public CreativeSplitPane() {
        init();
    }

    public CreativeSplitPane(int newOrientation) {
        super(newOrientation);
        init();
    }

    public CreativeSplitPane(int newOrientation, boolean newContinuousLayout) {
        super(newOrientation, newContinuousLayout);
        init();
    }

    public CreativeSplitPane(int newOrientation, Component newLeftComponent, Component newRightComponent) {
        super(newOrientation, newLeftComponent, newRightComponent);
        init();
    }

    public CreativeSplitPane(int newOrientation, boolean newContinuousLayout, Component newLeftComponent, Component newRightComponent) {
        super(newOrientation, newContinuousLayout, newLeftComponent, newRightComponent);
        init();
    }

    void init() {
        this.setOpaque(false);
    }

    @Override
    protected void paintComponent(Graphics g) {
        //super.paintComponent(g);
        if(1==0) {
        Graphics2D g2 = (Graphics2D) g.create();
        Color c1 = Color.white;
        float[] hsb = new float[3];
        int grayLevel = 212;
        Color.RGBtoHSB(grayLevel, grayLevel, grayLevel, hsb);
        Color c2 = Color.getHSBColor(hsb[0], hsb[1], hsb[2]);
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        GradientPaint gp = new GradientPaint(0, 0,
                c1,
                getWidth() * 2 / 3, getHeight(),
                c2);
        g2.setPaint(gp);
        g2.fillRect(0, 0, getWidth(), getHeight());

        //g2.setColor(Color.red);
        //g2.drawString("Creative split pane", 10, getHeight() / 2 + 20 / 2);
        if (getBgImage() != null && !BG_POSITION.none.equals(bgPosition)) {
            int x = 0;
            int y = 0;

            x = getWidth() / 2 - getBgImage().getIconWidth() / 2;
            if (x < 0) {
                x = 0;
            }
            switch (bgPosition) {
                case center: {
                    y = getHeight() / 2 - getBgImage().getIconHeight() / 2;
                    if (y < 0) {
                        y = 0;
                    }
                }
                break;
                case top:
                    y = 0;
                    break;
                case bottom:
                    y = getHeight() - getBgImage().getIconHeight();
                    if (y < 0) {
                        y = 0;
                    }
                    break;
                case stretch: {
                    x = 0;
                    y = 0;
                    double sx = 1.0 * getWidth() / getBgImage().getIconWidth();
                    double sy = 1.0 * getHeight() / getBgImage().getIconHeight();
                    g2.scale(sx, sy);
                }
                break;
            }
            g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.2f));

            g2.drawImage(getBgImage().getImage(), x, y, getBgImage().getIconWidth(), getBgImage().getIconHeight(), this);
        }
        }
        super.paintComponent(g);
    }

    @Override
    protected void paintChildren(Graphics g) {
        super.paintChildren(g);
    }

    /**
     * @return the bgImage
     */
    public ImageIcon getBgImage() {
        return bgImage;
    }

    /**
     * @param bgImage the bgImage to set
     */
    public void setBgImage(ImageIcon bgImage, BG_POSITION pos) {
        this.bgImage = bgImage;
        if (pos != null) {
            bgPosition = pos;
        }
    }
}
