/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.vc;

import ssg.tools.easyconfig.data.handler.IConfigurationHandler;
import ssg.tools.easyconfig.models.ConfigurationTreeModel;
import ssg.tools.easyconfig.data.NSetting;
import ssg.tools.easyconfig.data.NSettingSource;
import ssg.tools.easyconfig.models.NSettingTableModel;
import ssg.tools.easyconfig.vc.behaviours.ChangeConfigModeTreeMouseListener;
import ssg.tools.easyconfig.vc.behaviours.TreeExpanderKeyListener;
import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

/**
 *
 * @author ssg
 */
public class DefaultConfiguratorPanel extends javax.swing.JPanel implements IConfiguratorPanel {

    public static final String TEMPLATE_EXTENSION = ".config.template";

    /**
     * Creates new form DefaultConfiguratorPanel
     */
    public DefaultConfiguratorPanel() {
        initComponents();
        initCustom();
        this.setOpaque(false);
        this.jSplitPane1.setOpaque(false);
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jSplitPane1 = new CreativeSplitPane();
        jScrollPane1 = new CreativeScrollPane();
        jTree1 = new CreativeTree();
        jScrollPane2 = new CreativeScrollPane();
        jTable1 = new CreativeTable();
        jButtonUseDefaults = new javax.swing.JButton();
        jButtonLoaded = new javax.swing.JButton();
        jButtonApply = new javax.swing.JButton();
        jButtonSaveAsTemplate = new javax.swing.JButton();
        jCheckBoxViewAll = new javax.swing.JCheckBox();

        jSplitPane1.setBorder(null);
        jSplitPane1.setDividerLocation(200);
        jSplitPane1.setDividerSize(3);

        jScrollPane1.setViewportView(jTree1);

        jSplitPane1.setLeftComponent(jScrollPane1);

        jTable1.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane2.setViewportView(jTable1);

        jSplitPane1.setRightComponent(jScrollPane2);

        jButtonUseDefaults.setText("Use defaults");
        jButtonUseDefaults.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonUseDefaultsActionPerformed(evt);
            }
        });

        jButtonLoaded.setText("Restore");
        jButtonLoaded.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonLoadedActionPerformed(evt);
            }
        });

        jButtonApply.setText("Apply");
        jButtonApply.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonApplyActionPerformed(evt);
            }
        });

        jButtonSaveAsTemplate.setText("Save as template");
        jButtonSaveAsTemplate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonSaveAsTemplateActionPerformed(evt);
            }
        });

        jCheckBoxViewAll.setText("View all");
        jCheckBoxViewAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBoxViewAllActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jButtonApply)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jButtonLoaded)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jButtonUseDefaults)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jButtonSaveAsTemplate)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jCheckBoxViewAll)
                .addContainerGap())
            .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 558, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 305, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButtonUseDefaults)
                    .addComponent(jButtonLoaded)
                    .addComponent(jButtonApply)
                    .addComponent(jButtonSaveAsTemplate)
                    .addComponent(jCheckBoxViewAll)))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void jButtonLoadedActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonLoadedActionPerformed
        for (NSettingSource nss : config.getSettings()) {
            for (NSetting ns : nss.getSettings()) {
                if (!ns.isReadOnly()) {
                    ns.setValue(ns.getLoadedValue());
                }
            }
        }
        jTree1.repaint();
        updateTableFromTreeSelection(jTree1);
    }//GEN-LAST:event_jButtonLoadedActionPerformed

    private void jCheckBoxViewAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBoxViewAllActionPerformed
        updateTableFromTreeSelection(jTree1);
    }//GEN-LAST:event_jCheckBoxViewAllActionPerformed

    private void jButtonApplyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonApplyActionPerformed
        try {
            config.saveSettings();
        } catch (IOException ioex) {
            ioex.printStackTrace();
        }
    }//GEN-LAST:event_jButtonApplyActionPerformed

    private void jButtonUseDefaultsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonUseDefaultsActionPerformed
        for (NSettingSource nss : config.getSettings()) {
            for (NSetting ns : nss.getSettings()) {
                if (!ns.isReadOnly() && ns.getDefaultValue() != null) {
                    ns.setValue(ns.getDefaultValue());
                }
            }
        }
        jTree1.repaint();
        updateTableFromTreeSelection(jTree1);
    }//GEN-LAST:event_jButtonUseDefaultsActionPerformed

    private void jButtonSaveAsTemplateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonSaveAsTemplateActionPerformed
        try {
            // TODO: choose file to store settings
            config.saveConfigurationTemplate(config.getSettings(), new FileOutputStream(new File("template" + TEMPLATE_EXTENSION)), "*");
        } catch (IOException ioex) {
            ioex.printStackTrace();
        }

    }//GEN-LAST:event_jButtonSaveAsTemplateActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButtonApply;
    private javax.swing.JButton jButtonLoaded;
    private javax.swing.JButton jButtonSaveAsTemplate;
    private javax.swing.JButton jButtonUseDefaults;
    private javax.swing.JCheckBox jCheckBoxViewAll;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JTable jTable1;
    private javax.swing.JTree jTree1;
    // End of variables declaration//GEN-END:variables
    private IConfigurationHandler config;
    private ConfigurationTreeModel treeModel;
    private NSettingTableModel tableModel = new NSettingTableModel(null, false);
    private String helpText = "Drag'n'Drop WAR file(s) or web application folder(s) on this window.\n"
                +"Once applications are added choose any from drop list below.\n"
                +"\n"
                +"Drag'n'Drop settings template file to apply it to selected application.\n"
                +"\n"
                +"Select in tree root, file or setting (in any combination) to see them in table.\n"
                +"Edit values in table. Modified settings are highlighted with red.\n"
                +"Click on 'Apply changes' to write changes to WAR or files in folder.\n"
                +"Click on 'Save as template' to remember settings in settings template.";


    void initCustom() {
        this.setOpaque(false);
        jButtonApply.setOpaque(false);
        jButtonLoaded.setOpaque(false);
        jButtonSaveAsTemplate.setOpaque(false);
        jButtonUseDefaults.setOpaque(false);
        jCheckBoxViewAll.setOpaque(false);

        jScrollPane1.setViewportBorder(null);
        jScrollPane1.getViewport().setBorder(null);
        jScrollPane1.getViewport().getInsets().set(0, 0, 0, 0);
        jScrollPane2.setViewportBorder(null);
        jScrollPane2.getViewport().setBorder(null);
        jScrollPane2.getViewport().getInsets().set(0, 0, 0, 0);
        jSplitPane1.setBorder(null);
        jSplitPane1.getInsets().set(0, 0, 0, 0);
        ((JComponent) jSplitPane1.getLeftComponent()).setBorder(null);
        ((JComponent) jSplitPane1.getLeftComponent()).getInsets().set(0, 0, 0, 0);
        ((JComponent) jSplitPane1.getRightComponent()).setBorder(null);
        ((JComponent) jSplitPane1.getRightComponent()).getInsets().set(0, 0, 0, 0);


        jTree1.setCellRenderer(new DefaultTreeCellRenderer() {

            @Override
            public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
                Component c = super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
                if (c instanceof JLabel) {
                    try {
                        JLabel lbl = (JLabel) c;
                        Object obj = ConfigurationTreeModel.getUserObject(value);
                        if (obj instanceof IConfigurationHandler) {
                            IConfigurationHandler config = (IConfigurationHandler) obj;
                            if (config.isModified()) {
                                lbl.setToolTipText("<html><b>" + config.getLocation() + "</b></html>");
                            } else {
                                lbl.setToolTipText(config.getLocation());
                            }
                        } else if (obj instanceof NSettingSource) {
                            NSettingSource nss = (NSettingSource) obj;
                            if (nss.isModified()) {
                                lbl.setText("<html><b>" + nss + "</b></html>");
                            }
                            //lbl.setPreferredSize(new Dimension(10000, lbl.getPreferredSize().height));
                        } else if (obj instanceof NSetting) {
                            NSetting ns = (NSetting) obj;
                            if (ns.getValue() != null && ns.isModified()) {
                                lbl.setText("<html><b>" + ns.getName() + "</b> = <font color=red>" + ns.getValue() + "</font></html>");
                            } else {
                                lbl.setText(ns.getName() + " = " + ns.getValue());
                            }
                        }
                        Dimension oldD = lbl.getPreferredSize();
                        lbl.setPreferredSize(new Dimension(Math.max(oldD.width, 300), lbl.getPreferredSize().height));
                    } catch (Throwable th) {
                        th.printStackTrace();
                    }
                }
                return c;
            }
        });
        jTree1.addTreeSelectionListener(new TreeSelectionListener() {

            public void valueChanged(TreeSelectionEvent e) {
                JTree tree = (JTree) e.getSource();
                updateTableFromTreeSelection(tree);
            }
        });
        jTree1.addMouseListener(new ChangeConfigModeTreeMouseListener());
        jTree1.addKeyListener(new TreeExpanderKeyListener());

//        this.jSplitPane1.setOpaque(true);
//        this.jScrollPane1.setOpaque(true);
//        this.jScrollPane2.setOpaque(true);
//        this.jTree1.setOpaque(true);
//        jTable1.setOpaque(true);
        jTable1.setDefaultRenderer(Object.class, new DefaultTableCellRenderer() {

            @Override
            public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
                if (column == 1 && table.getModel() instanceof NSettingTableModel) {
                    NSettingTableModel tm = (NSettingTableModel) table.getModel();
                    if (!tm.isCellEditable(row, column)) {
                        setForeground(Color.darkGray);
                    } else if (tm.IsModifiedCell(row, column)) {
                        setBackground(Color.pink);
                    }
                } else {
                    setForeground(null);
                    setBackground(null);
                }
                Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                if (jSplitPane1 instanceof CreativeSplitPane
                        && ((CreativeSplitPane) jSplitPane1).getBgImage() != null
                        && c instanceof JComponent) {
                    ((JComponent) c).setOpaque(isSelected);
                    //((JComponent) c).setBorder(javax.swing.BorderFactory.createEtchedBorder());
                    //((JComponent) c).setBorder(javax.swing.BorderFactory.createLineBorder(getBackground()));
                    ((JComponent) c).setBorder(javax.swing.BorderFactory.createLineBorder(getBackground()));
                }
                return c;
            }
        });
        //((JComponent)jTable1.getDefaultRenderer(Object.class)).setOpaque(false);

        jTable1.setModel(tableModel);
        jTable1.getModel().addTableModelListener(new TableModelListener() {

            public void tableChanged(TableModelEvent e) {
                JTable table = jTable1;//(JTable) e.getSource();
                if (table.getModel() instanceof NSettingTableModel) {
                    NSettingTableModel model = (NSettingTableModel) table.getModel();
                    if (e.getColumn() == 1 && e.getFirstRow() == e.getLastRow()) {
                        NSetting ns = model.getSetting(e.getFirstRow());
                        if (ns != null) {
                            JTree tree = jTree1;
                            TreeNode tn = (TreeNode) treeModel.getChild(ns.getSource(), ns.getSource().getSettings().indexOf(ns));
                            for (int i = 0; i < tree.getRowCount(); i++) {
                                TreePath tp = tree.getPathForRow(i);
                                Object obj = ConfigurationTreeModel.getUserObject(tp);
                                if (obj != null && obj.equals(ns)) {
                                    treeModel.clearCache(obj);
                                    treeModel.nodeChanged((TreeNode) tp.getLastPathComponent());
                                    treeModel.nodeChanged((TreeNode) tp.getParentPath().getLastPathComponent());
                                }
                                tree.revalidate();
                                tree.repaint();
                            }
//                            if (tn != null) {
//                                treeModel.nodeStructureChanged((TreeNode)treeModel.getRoot());
//                                //treeModel.nodeChanged(tn);
//                            }
                        }
                    }
                }
            }
        });
        setConfig(null);
    }

    /**
     * @return the config
     */
    public IConfigurationHandler getConfig() {
        return config;
    }

    /**
     * @param config the config to set
     */
    public void setConfig(IConfigurationHandler config) {
        this.config = config;
        treeModel = new ConfigurationTreeModel(config);
        jTree1.setModel(treeModel);
        try {
            for (int i = 0; i < jTree1.getRowCount(); i++) {
                jTree1.expandRow(i);
            }
            jTree1.getSelectionModel().setSelectionPath(jTree1.getPathForRow(0));
        } catch (Throwable th) {
        }
        updateTableFromTreeSelection(jTree1);
    }

    public void updateTableFromTreeSelection(JTree tree) {
        List<NSetting> settings = new ArrayList<NSetting>();
        for (int i = 0; i < tree.getRowCount(); i++) {
            if (tree.isRowSelected(i)) {
                TreePath path = tree.getPathForRow(i);
                Object obj = ConfigurationTreeModel.getUserObject(path);
                if (obj instanceof IConfigurationHandler) {
                    // add all settings
                    for (NSettingSource nss : ((IConfigurationHandler) obj).getSettings()) {
                        for (NSetting ns : nss.getSettings()) {
                            if (!settings.contains(ns)) {
                                settings.add(ns);
                            }
                        }
                    }
                } else if (obj instanceof NSettingSource) {
                    // add all source settings
                    for (NSetting ns : ((NSettingSource) obj).getSettings()) {
                        if (!settings.contains(ns)) {
                            settings.add(ns);
                        }
                    }
                } else if (obj instanceof NSetting) {
                    // add this setting
                    if (!settings.contains(obj)) {
                        settings.add((NSetting) obj);
                    }
                } else if (obj instanceof String) {
                    // add this setting
                    for (int j = 0; j < treeModel.getChildCount(obj); j++) {
                        Object obj2 = ConfigurationTreeModel.getUserObject(treeModel.getChild(obj, j));
                        if (obj2 instanceof NSetting && !settings.contains(obj2)) {
                            settings.add((NSetting) obj2);
                        }
                    }
                }
            }
        }
        tableModel.setSettings(settings, this.jCheckBoxViewAll.isSelected());
    }

    @Override
    protected void paintChildren(Graphics g) {
        super.paintChildren(g);
        if (config == null) {
            String[] texts = (getHelpText()!=null) ? getHelpText().split("\n") : "".split("\n");

            int lineSpacing = 25;
            int baseX = 10;
            int y = this.getHeight() / 2 - texts.length * lineSpacing / 2;
            {
                Graphics2D g2 = (Graphics2D) g.create();
                g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.3f));
                g2.setColor(Color.blue);
                BasicStroke lineStroke = new BasicStroke(3);
                g2.setStroke(lineStroke);
                Rectangle rect = new Rectangle(
                        baseX,
                        y - lineSpacing * 2,
                        this.getWidth() - baseX * 2,
                        (texts.length + 2) * lineSpacing + lineSpacing / 2);
                g2.fill(rect);

                g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                        RenderingHints.VALUE_ANTIALIAS_ON);
                GradientPaint gp = new GradientPaint(0, 0,
                        getBackground().brighter().brighter(),
                        0, rect.height,
                        getBackground().darker().darker().darker().darker());
                gp = new GradientPaint(0, rect.y,
                        getBackground().darker().darker().darker().darker(),
                        0, rect.height,
                        getBackground().brighter().brighter());
                g2.setPaint(gp);
                g2.fill(rect);

                g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.3f));
                g2.setColor(Color.black);
                g2.draw(rect);
            }
            {
                Graphics2D g2 = (Graphics2D) g.create();
                Font f = g2.getFont();
                f = Font.decode(f.getFontName() + "-BOLD-16");
                g2.setFont(f);
                for (String s : texts) {
                    g2.setColor(Color.LIGHT_GRAY);
                    g2.drawString(s, baseX + 10 + 1, y + 1);
                    g2.setColor(Color.black);
                    g2.drawString(s, baseX + 10, y);
                    y += lineSpacing;
                }
            }
        }
    }

    @Override
    protected void paintComponent(Graphics g) {
        //super.paintComponent(g);
        if (1 == 0) {
            Graphics2D g2 = (Graphics2D) g.create();
            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
            GradientPaint gp = new GradientPaint(0, 0,
                    getBackground().brighter().brighter().brighter(), 0, getHeight(),
                    getBackground().darker());
            g2.setPaint(gp);
            g2.fillRect(0, 0, getWidth(), getHeight());
        }
        super.paintComponent(g);
    }

    public void setBgImage(ImageIcon img, BG_POSITION pos) {
        ((CreativeSplitPane) this.jSplitPane1).setBgImage(img, pos);
    }

    /**
     * @return the helpText
     */
    public String getHelpText() {
        return helpText;
    }

    /**
     * @param helpText the helpText to set
     */
    public void setHelpText(String helpText) {
        this.helpText = helpText;
    }
}
