/*
 * Copyright 2012 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ssg.tools.easyconfig.vc.behaviours;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import javax.swing.JTree;
import javax.swing.tree.TreePath;

/**
 *
 * @author ssg
 */
public class TreeExpanderKeyListener implements KeyListener {

    public void keyTyped(KeyEvent e) {
    }

    public void keyPressed(KeyEvent e) {
        if (e.isControlDown()) {
            JTree tree = (JTree) e.getSource();
            int levelToProcess = -1;
            for (int i = 0; i < tree.getRowCount(); i++) {
                TreePath tp = tree.getPathForRow(i);
                int level = tp.getPathCount();

                // reset node level if sub-tree is over
                if (levelToProcess != -1 && level == levelToProcess) {
                    levelToProcess = -1;
                }

                // set node level to process if non yet
                if (tree.isRowSelected(i) && levelToProcess == -1) {
                    levelToProcess = level;
                }

                if (levelToProcess != -1) {
                    if (e.getKeyChar() == '+' || e.getKeyCode() == KeyEvent.VK_RIGHT) {
                        // expand subtree
                        if (tree.isExpanded(i)) {
                            continue;
                        } else {
                            tree.expandRow(i);
                        }
                    } else if (e.getKeyChar() == '-' || e.getKeyCode() == KeyEvent.VK_LEFT) {
                        // collapse subtree
                        if (tree.isCollapsed(i)) {
                            continue;
                        } else {
                            tree.collapseRow(i);
                        }
                    }
                }
            }

        }
    }

    public void keyReleased(KeyEvent e) {
    }
}
